import os

import time
from datetime import datetime
from ..utils.serializer import dumps, loads, dumps_str, loads_str
import signal

import asyncio
import logging
import contextlib
import importlib
import time 

from typing import List


from .message import TaskMessage
from .task import Task
from .enums import TaskStatus
from jettask.messaging.event_pool import EventPool
from ..executor.orchestrator import ProcessOrchestrator
from ..utils import gen_task_name
from ..scheduler import TaskScheduler
from ..exceptions import TaskTimeoutError, TaskExecutionError, TaskNotFoundError
from jettask.db.connector import get_sync_redis_client, get_async_redis_client

try:
    import uvloop
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    UVLOOP_AVAILABLE = True
except ImportError:
    UVLOOP_AVAILABLE = False

logger = logging.getLogger('app')

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(levelname)s - %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

if UVLOOP_AVAILABLE:
    logger.debug("Using uvloop for better performance")
else:
    logger.debug("uvloop not available, using default asyncio event loop")


class Jettask(object):

    def __init__(self, redis_url: str = None, include: list = None, max_connections: int = None,
                 consumer_config: dict = None, tasks=None,
                 redis_prefix: str = None, scheduler_config: dict = None, pg_url: str = None,
                 task_center=None, 
                 heartbeat_interval: float = 5.0, heartbeat_timeout: float = 15.0,
                 scanner_interval: float = 2) -> None:
        self._tasks = tasks or {}
        self._task_queues = {}  
        self.asyncio = False
        self.include = include or []

        self.task_center = None  
        self._task_center_config = None
        self._original_redis_url = redis_url
        self._original_pg_url = pg_url

        self.redis_url = redis_url or os.environ.get('JETTASK_REDIS_URL')
        self.pg_url = pg_url or os.environ.get('JETTASK_PG_URL')
        self.max_connections = max_connections if max_connections is not None else int(os.environ.get('JETTASK_MAX_CONNECTIONS', '500'))
        self.redis_prefix = redis_prefix or os.environ.get('JETTASK_REDIS_PREFIX', 'jettask')

        if not self.redis_url:
            raise ValueError(
                "必须提供 redis_url 参数！\n\n"
                "请通过以下任一方式配置:\n"
                "  1. 初始化时传参:\n"
                "     app = Jettask(redis_url='redis://localhost:6379/0')\n\n"
                "  2. 设置环境变量:\n"
                "     export JETTASK_REDIS_URL='redis://localhost:6379/0'\n\n"
                "  3. 在 .env 文件中配置:\n"
                "     JETTASK_REDIS_URL=redis://localhost:6379/0\n"
            )

        self.consumer_config = consumer_config or {}
        self.scheduler_config = scheduler_config or {}

        self.heartbeat_interval = heartbeat_interval  
        self.heartbeat_timeout = heartbeat_timeout    
        self.scanner_interval = scanner_interval      

        if task_center:
            self.mount_task_center(task_center)

        self.STATUS_PREFIX = f"{self.redis_prefix}:STATUS:"
        self.RESULT_PREFIX = f"{self.redis_prefix}:RESULT:"
        
        self._loads = loads
        self._dumps = dumps
        
        self.scheduler = None
        self._scheduler_db_url = None

        self._status_prefix = self.STATUS_PREFIX
        self._result_prefix = self.RESULT_PREFIX

        self.worker_state_manager = None

        self._worker_state = None


        self._cleanup_done = False
        self._should_exit = False
        self._worker_started = False
        self._handlers_registered = False

        self._registry = None
        self._registry_initialized = False

        self._pending_rate_limits = {}
   
    
    def _load_config_from_task_center(self):
        try:
            import asyncio
            try:
                loop = asyncio.get_running_loop()
                return False
            except RuntimeError:
                loop = asyncio.new_event_loop()
                if self.task_center:
                    if self.task_center._initialized:
                        config = self.task_center._config
                    else:
                        success = loop.run_until_complete(self.task_center.connect(asyncio=True))
                        if success:
                            config = self.task_center._config
                        else:
                            config = None
                else:
                    config = None
                loop.close()
            
            if config:
                redis_config = config.get('redis_config', {})
                pg_config = config.get('pg_config', {})
                if redis_config:
                    redis_host = redis_config.get('host', 'localhost')
                    redis_port = redis_config.get('port', 6379)
                    redis_password = redis_config.get('password')
                    redis_db = redis_config.get('db', 0)
                    
                    if redis_password:
                        self.redis_url = f"redis://:{redis_password}@{redis_host}:{redis_port}/{redis_db}"
                    else:
                        self.redis_url = f"redis://{redis_host}:{redis_port}/{redis_db}"
                    
                    logger.debug(f"从任务中心加载Redis配置: {redis_host}:{redis_port}/{redis_db}")
                
                if pg_config:
                    pg_host = pg_config.get('host', 'localhost')
                    pg_port = pg_config.get('port', 5432)
                    pg_user = pg_config.get('user', 'postgres')
                    pg_password = pg_config.get('password', '')
                    pg_database = pg_config.get('database', 'jettask')
                    
                    self.pg_url = f"postgresql://{pg_user}:{pg_password}@{pg_host}:{pg_port}/{pg_database}"
                    logger.debug(f"从任务中心加载PostgreSQL配置: {pg_host}:{pg_port}/{pg_database}")
                
                self._task_center_config = config
                
                if self.task_center and self.task_center.redis_prefix != "jettask":
                    self.redis_prefix = self.task_center.redis_prefix
                    self.STATUS_PREFIX = f"{self.redis_prefix}:STATUS:"
                    self.RESULT_PREFIX = f"{self.redis_prefix}:RESULT:"
                
                if hasattr(self, '_redis'):
                    delattr(self, '_redis')
                if hasattr(self, '_async_redis'):
                    delattr(self, '_async_redis')
                if hasattr(self, '_ep'):
                    delattr(self, '_ep')
                
        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.warning(f"从任务中心加载配置失败，使用手动配置: {e}")
            self.redis_url = self._original_redis_url
            self.pg_url = self._original_pg_url
    
    def mount_task_center(self, task_center):
        self.task_center = task_center
        
        if task_center and task_center._initialized:
            if task_center.redis_config:
                redis_url = task_center.get_redis_url()
                if redis_url:
                    self.redis_url = redis_url
                    
            if task_center.pg_config:
                pg_url = task_center.get_pg_url()
                if pg_url:
                    self.pg_url = pg_url
            
            self.redis_prefix = task_center.redis_prefix
            self.STATUS_PREFIX = f"{self.redis_prefix}:STATUS:"
            self.RESULT_PREFIX = f"{self.redis_prefix}:RESULT:"
            self.QUEUE_PREFIX = f"{self.redis_prefix}:QUEUE:"
            self.STREAM_PREFIX = f"{self.redis_prefix}:STREAM:"
            self.TASK_PREFIX = f"{self.redis_prefix}:TASK:"
            self.SCHEDULER_PREFIX = f"{self.redis_prefix}:SCHEDULED:"
            self.LOCK_PREFIX = f"{self.redis_prefix}:LOCK:"
            
            self._task_center_config = {
                'redis_config': task_center.redis_config,
                'pg_config': task_center.pg_config,
                'namespace_name': task_center.namespace_name,
                'version': task_center.version
            }
    
    
    def _setup_cleanup_handlers(self):
        if self._handlers_registered:
            return
        
        self._handlers_registered = True
        
        def signal_cleanup_handler(signum=None, frame=None):
            if self._cleanup_done:
                return
            if self._worker_started:
                logger.debug("Received shutdown signal, cleaning up...")
            self.cleanup()
            if signum:
                self._should_exit = True
        
        def atexit_cleanup_handler():
            if self._cleanup_done:
                return
            self.cleanup()
        
        if hasattr(signal, 'SIGTERM'):
            signal.signal(signal.SIGTERM, signal_cleanup_handler)
        if hasattr(signal, 'SIGINT'):
            signal.signal(signal.SIGINT, signal_cleanup_handler)
        
        import atexit
        atexit.register(atexit_cleanup_handler)
    
    def cleanup(self):
        if self._cleanup_done:
            return
        self._cleanup_done = True
        
        if self._worker_started:
   
            
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()
            

        else:
            if hasattr(self, 'ep') and self.ep:
                self.ep.cleanup()


    @property
    def async_redis(self):
        if self.task_center and self.task_center.is_enabled and not self._task_center_config:
            self._load_config_from_task_center()

        return get_async_redis_client(
            redis_url=self.redis_url,
            decode_responses=True,
            max_connections=self.max_connections,
            socket_timeout=None  
        )

    @property
    def redis(self):

        return get_sync_redis_client(
            redis_url=self.redis_url,
            decode_responses=True,
            max_connections=self.max_connections
        )

    @property
    def binary_redis(self):
        return get_sync_redis_client(
            redis_url=self.redis_url,
            decode_responses=False,
            max_connections=self.max_connections
        )

    @property
    def async_binary_redis(self):
        return get_async_redis_client(
            redis_url=self.redis_url,
            decode_responses=False,
            max_connections=self.max_connections,
            socket_timeout=None
        )

    @property
    def worker_state(self):
        if self._worker_state is None:
            from jettask.worker.lifecycle import WorkerManager
            self._worker_state = WorkerManager(
                redis_client=self.redis,
                async_redis_client=self.async_redis,
                redis_prefix=self.redis_prefix
            )
            logger.debug("Initialized WorkerManager for app")
        return self._worker_state

    @property
    def ep(self):
        name = "_ep"
        if hasattr(self, name):
            ep = getattr(self, name)
        else:
            consumer_config = self.consumer_config.copy() if self.consumer_config else {}
            consumer_config['redis_prefix'] = self.redis_prefix


            task_event_queues = {}

            ep = EventPool(
                redis_client=self.redis,
                async_redis_client=None,  
                task_event_queues=task_event_queues,
                tasks=self._tasks,
                queue_registry=None,  
                offline_recovery=self.worker_state_manager,  
                redis_url=self.redis_url,
                consumer_config=consumer_config,
                redis_prefix=self.redis_prefix,
                app=self
            )
            setattr(self, name, ep)
        return ep

    @property
    def registry(self):
        if self._registry_initialized and self._registry:
            return self._registry

        from ..messaging.registry import QueueRegistry

        self._registry = QueueRegistry(
            redis_client=self.redis,  
            async_redis_client=self.async_redis,  
            redis_prefix=self.redis_prefix
        )

        self._registry_initialized = True
        return self._registry

    def clear(self):
        if hasattr(self, "process"):
            delattr(self, "process")
        if hasattr(self, "_ep"):
            delattr(self, "_ep")
        if hasattr(self, "_registry"):
            self._registry = None
            self._registry_initialized = False

    def get_task_by_name(self, name: str) -> Task:
        task = self._tasks.get(name)
        if task:
            return task
        
        if '.' not in name:
            for task_key, task_obj in self._tasks.items():
                if '.' in task_key:
                    _, func_name = task_key.rsplit('.', 1)
                    if func_name == name:
                        return task_obj
                elif task_key == name:
                    return task_obj
        
        return None

    def get_task_config(self, task_name: str) -> dict:
        task = self.get_task_by_name(task_name)
        if not task:
            return None

        return {
            'auto_ack': getattr(task, 'auto_ack', True),
            'queue': getattr(task, 'queue', None),
            'timeout': getattr(task, 'timeout', None),
            'max_retries': getattr(task, 'max_retries', 0),
            'retry_delay': getattr(task, 'retry_delay', None),
        }

    def include_module(self, modules: list):
        self.include += modules

    def _task_from_fun(
        self, fun, name=None, base=None, queue=None, bind=False, retry_config=None, rate_limit=None, auto_ack=True, **options
    ) -> Task:
        name = name or gen_task_name(fun.__name__, fun.__module__)
        base = base or Task


        if name not in self._tasks:
            run = staticmethod(fun)
            task: Task = type(
                fun.__name__,
                (base,),
                dict(
                    {
                        "app": self,
                        "name": name,
                        "run": run,
                        "queue": queue,
                        "retry_config": retry_config,  
                        "rate_limit": rate_limit,  
                        "auto_ack": auto_ack,  
                        "_decorated": True,
                        "__doc__": fun.__doc__,
                        "__module__": fun.__module__,
                        "__annotations__": fun.__annotations__,
                        "__wrapped__": run,
                    },
                    **options,
                ),
            )()
            task.bind_app(self)
            with contextlib.suppress(AttributeError):
                task.__qualname__ = fun.__qualname__
            self._tasks[task.name] = task

            if queue:
                self._task_queues[name] = queue

            if rate_limit:
                if isinstance(rate_limit, int) and rate_limit > 0:
                    self._register_rate_limit(name, rate_limit)
                elif hasattr(rate_limit, 'to_dict'):
                    self._register_rate_limit_config(name, rate_limit)
            else:
                from jettask.utils.rate_limit.limiter import RateLimiterManager

                RateLimiterManager.unregister_rate_limit_config(
                    redis_client=self.redis,
                    task_name=name,
                    redis_prefix=self.redis_prefix
                )
        else:
            task = self._tasks[name]
        return task

    def _register_rate_limit(self, task_name: str, qps_limit: int):
        from jettask.utils.rate_limit.config import QPSLimit

        config = QPSLimit(qps=qps_limit)
        self._pending_rate_limits[task_name] = config
        logger.debug(f"存储任务 {task_name} 的 QPS 限流配置，将在 start() 时注册")

    def _register_rate_limit_config(self, task_name: str, config):
        self._pending_rate_limits[task_name] = config
        logger.debug(f"存储任务 {task_name} 的限流配置，将在 start() 时注册")

    def _apply_pending_rate_limits(self):
        if not self._pending_rate_limits:
            return

        from jettask.utils.rate_limit.limiter import RateLimiterManager

        for task_name, config in self._pending_rate_limits.items():
            try:
                RateLimiterManager.register_rate_limit_config(
                    redis_client=self.redis,
                    task_name=task_name,
                    config=config,
                    redis_prefix=self.redis_prefix
                )
                logger.debug(f"已注册任务 {task_name} 的限流配置")
            except Exception as e:
                logger.error(f"注册任务 {task_name} 的限流配置失败: {e}")

        logger.debug(f"已注册 {len(self._pending_rate_limits)} 个任务的限流配置")

    def _get_queues_from_tasks(self, task_names: List[str]) -> List[str]:
        queues = []
        for task_name in task_names:
            if task_name not in self._task_queues:
                available_tasks = list(self._task_queues.keys())
                raise ValueError(
                    f"任务 '{task_name}' 不存在或未配置队列。\n"
                    f"已注册的任务: {', '.join(available_tasks) if available_tasks else '(无)'}"
                )

            queue = self._task_queues[task_name]

            if queue not in queues:
                queues.append(queue)

        logger.debug(f"从 {len(task_names)} 个任务获取到 {len(queues)} 个队列: {queues}")
        return queues

    def get_tasks_by_queue(self, queue_name: str) -> List[str]:
        return [task_name for task_name, queue in self._task_queues.items() if queue == queue_name]

    def task(
        self,
        name: str = None,
        queue: str = None,
        base: Task = None,
        max_retries: int = 0,
        retry_backoff: bool = True,  
        retry_backoff_max: float = 60,  
        retry_on_exceptions: tuple = None,  
        rate_limit: int = None,  
        auto_ack: bool = True,  
        *args,
        **kwargs,
    ):
        def _create_task_cls(fun):
            retry_config = None
            if max_retries > 0:
                retry_config = {
                    'max_retries': max_retries,
                    'retry_backoff': retry_backoff,
                    'retry_backoff_max': retry_backoff_max,
                }
                if retry_on_exceptions:
                    retry_config['retry_on_exceptions'] = [
                        exc if isinstance(exc, str) else exc.__name__
                        for exc in retry_on_exceptions
                    ]

            return self._task_from_fun(
                fun,
                name,
                base,
                queue,
                retry_config=retry_config,
                rate_limit=rate_limit,
                auto_ack=auto_ack,  
                *args,
                **kwargs
            )

        return _create_task_cls
    
    def include_router(self, router, prefix: str = None):
        from ..task.router import TaskRouter
        
        if not isinstance(router, TaskRouter):
            raise TypeError(f"Expected TaskRouter, got {type(router)}")
        
        tasks = router.get_tasks()
        
        for task_name, task_config in tasks.items():
            config = task_config.copy()
            
            if prefix:
                if config.get('name'):
                    config['name'] = f"{prefix}.{config['name']}"
                else:
                    config['name'] = f"{prefix}.{task_name}"
            
            func = config.pop('func')
            name = config.pop('name', task_name)
            queue = config.pop('queue', None)
            
            retry_config = {}
            if 'max_retries' in config:
                retry_config['max_retries'] = config.pop('max_retries', 0)
            if 'retry_delay' in config:
                retry_config['retry_backoff_max'] = config.pop('retry_delay', 60)
            
            self._task_from_fun(
                func,
                name=name,
                queue=queue,
                retry_config=retry_config if retry_config else None,
                **config
            )
    
    def send_tasks(self, messages: list, asyncio: bool = False):
        if asyncio:
            return self._send_tasks_async(messages)
        else:
            return self._send_tasks_sync(messages)

    def ack(self, ack_items: list):
        if not ack_items:
            return

        if not hasattr(self, '_executor_core') or not self._executor_core:
            logger.warning("ACK can only be called in worker context")
            return

        for item in ack_items:
            if isinstance(item, dict):
                queue = item['queue']
                event_id = item['event_id']
                group_name = item.get('group_name')
                offset = item.get('offset')
            elif isinstance(item, (tuple, list)):
                if len(item) >= 3:
                    queue, event_id, group_name = item[0], item[1], item[2]
                    offset = item[3] if len(item) > 3 else None
                elif len(item) == 2:
                    logger.error(
                        f"Invalid ACK item format: {item}. "
                        f"group_name is required. Use (queue, event_id, group_name) or ctx.acks([event_ids])"
                    )
                    continue
                else:
                    logger.error(f"Invalid ACK item format: {item}")
                    continue
            else:
                logger.error(f"Invalid ACK item type: {type(item)}")
                continue

            if not group_name:
                logger.error(
                    f"group_name is required for ACK. item={item}. "
                    f"Use ctx.acks([event_ids]) for automatic group_name injection."
                )
                continue

            self._executor_core.pending_acks.append((queue, event_id, group_name, offset))
            logger.debug(f"Added to pending_acks: queue={queue}, event_id={event_id}, group_name={group_name}")

        if len(self._executor_core.pending_acks) >= 100:
            import asyncio
            try:
                loop = asyncio.get_running_loop()
                asyncio.create_task(self._executor_core._flush_all_buffers())
            except RuntimeError:
                pass

    def _send_tasks_sync(self, messages: list):
        if not messages:
            return []

        results = []

        queue_messages = {}
        for msg in messages:
            if isinstance(msg, dict):
                msg = TaskMessage.from_dict(msg)
            elif not isinstance(msg, TaskMessage):
                raise ValueError(f"Invalid message type: {type(msg)}. Expected TaskMessage or dict")

            msg.validate()

            actual_queue = msg.queue
            if msg.priority is not None:
                actual_queue = f"{msg.queue}:{msg.priority}"
                msg.queue = actual_queue

            if actual_queue not in queue_messages:
                queue_messages[actual_queue] = []
            queue_messages[actual_queue].append(msg)

        for queue, queue_msgs in queue_messages.items():
            batch_results = self._send_batch_messages_sync(queue, queue_msgs)
            results.extend(batch_results)

        return results

    async def _send_tasks_async(self, messages: list):
        if not messages:
            return []

        results = []

        queue_messages = {}
        for msg in messages:
            if isinstance(msg, dict):
                msg = TaskMessage.from_dict(msg)
            elif not isinstance(msg, TaskMessage):
                raise ValueError(f"Invalid message type: {type(msg)}. Expected TaskMessage or dict")

            msg.validate()

            actual_queue = msg.queue
            if msg.priority is not None:
                actual_queue = f"{msg.queue}:{msg.priority}"
                msg.queue = actual_queue

            if actual_queue not in queue_messages:
                queue_messages[actual_queue] = []
            queue_messages[actual_queue].append(msg)

        for queue, queue_msgs in queue_messages.items():
            batch_results = await self._send_batch_messages_async(queue, queue_msgs)
            results.extend(batch_results)

        return results
    
    def _send_batch_messages_sync(self, queue: str, messages: list) -> list:
        from ..utils.serializer import dumps_str

        all_messages = []

        for msg in messages:
            msg_dict = msg.to_dict()

            if msg.delay and msg.delay > 0:
                current_time = time.time()
                msg_dict['execute_at'] = current_time + msg.delay
                msg_dict['is_delayed'] = 1

            all_messages.append(msg_dict)

        results = []

        if all_messages:
            batch_results = self.ep._batch_send_event_sync(
                self.ep.get_prefixed_queue_name(queue),
                [{'data': dumps_str(msg)} for msg in all_messages],
                self.ep.get_redis_client(asyncio=False, binary=True).pipeline()
            )
            results.extend(batch_results)

        return results

    async def _send_batch_messages_async(self, queue: str, messages: list) -> list:
        from ..utils.serializer import dumps_str

        all_messages = []

        for msg in messages:
            msg_dict = msg.to_dict()

            if msg.delay and msg.delay > 0:
                current_time = time.time()
                msg_dict['execute_at'] = current_time + msg.delay
                msg_dict['is_delayed'] = 1

            all_messages.append(msg_dict)

        results = []
        if all_messages:
            batch_results = await self.ep._batch_send_event(
                self.ep.get_prefixed_queue_name(queue),
                [{'data': dumps_str(msg)} for msg in all_messages],
                self.ep.get_redis_client(asyncio=True, binary=True).pipeline()
            )
            results.extend(batch_results)

        return results

    def _get_task_names_from_queue(self, queue: str, task_name: str = None) -> list:
        if task_name is not None:
            return [task_name]

        base_queue = queue.split(':')[0] if ':' in queue else queue

        import asyncio
        try:
            loop = asyncio.get_running_loop()
            task_names = asyncio.create_task(self.registry.get_task_names_by_queue(base_queue))
            raise RuntimeError("Cannot call sync method from async context")
        except RuntimeError:
            task_names = asyncio.run(self.registry.get_task_names_by_queue(base_queue))

        return list(task_names) if task_names else []

    def get_result(self, event_id: str, queue: str, task_name: str = None,
                   delete: bool = False, asyncio: bool = False,
                   delayed_deletion_ex: int = None, wait: bool = False,
                   timeout: int = 300, poll_interval: float = 0.5):
        return_single = task_name is not None

        task_names = self._get_task_names_from_queue(queue, task_name)

        if not task_names:
            if asyncio:
                async def _return_empty_list():
                    return []
                return _return_empty_list()
            else:
                return []

        if asyncio:
            return self._get_results_async(event_id, queue, task_names, delete,
                                          delayed_deletion_ex, wait, timeout, poll_interval, return_single)
        else:
            return self._get_results_sync(event_id, queue, task_names, delete,
                                         delayed_deletion_ex, wait, timeout, poll_interval, return_single)

    def get_queue_position(self, event_id: str, queue: str, task_name: str = None, asyncio: bool = False):
        return_single = task_name is not None

        task_names = self._get_task_names_from_queue(queue, task_name)

        if not task_names:
            if asyncio:
                async def _return_empty_list():
                    return []
                return _return_empty_list()
            else:
                return []

        if asyncio:
            return self._get_queue_positions_async(event_id, queue, task_names, return_single)
        else:
            return self._get_queue_positions_sync(event_id, queue, task_names, return_single)

    def _get_queue_positions_sync(self, event_id: str, queue: str, task_names: list, return_single: bool):
        results = []

        prefixed_queue = f"{self.redis_prefix}:QUEUE:{queue}"

        try:
            stream_data = self.binary_redis.xrange(prefixed_queue, min=event_id, max=event_id, count=1)

            if not stream_data:
                for task_name in task_names:
                    results.append({
                        "task_name": task_name,
                        "error": "Task not found in stream"
                    })
                if return_single:
                    return results[0] if results else None
                return results

            message_id, message_data = stream_data[0]

            task_offset = None
            for key, value in message_data.items():
                if isinstance(key, bytes):
                    key = key.decode('utf-8')
                if key == 'offset':
                    if isinstance(value, bytes):
                        task_offset = int(value.decode('utf-8'))
                    else:
                        task_offset = int(value)
                    break

            if task_offset is None:
                for task_name in task_names:
                    results.append({
                        "task_name": task_name,
                        "error": "Offset not found in task data"
                    })
                if return_single:
                    return results[0] if results else None
                return results

        except Exception as e:
            for task_name in task_names:
                results.append({
                    "task_name": task_name,
                    "error": f"Failed to read from stream: {str(e)}"
                })
            if return_single:
                return results[0] if results else None
            return results

        read_offsets_key = f"{self.redis_prefix}:READ_OFFSETS"
        task_offsets_key = f"{self.redis_prefix}:TASK_OFFSETS"

        base_queue = queue.split(':')[0] if ':' in queue else queue

        offset_keys = [f"{base_queue}:{task_name}" for task_name in task_names]

        try:
            pipeline = self.redis.pipeline()
            pipeline.hmget(read_offsets_key, offset_keys)
            pipeline.hmget(task_offsets_key, offset_keys)
            read_offsets_list, task_offsets_list = pipeline.execute()
        except Exception as e:
            for task_name in task_names:
                results.append({
                    "task_name": task_name,
                    "error": f"Failed to read offsets: {str(e)}"
                })
            if return_single:
                return results[0] if results else None
            return results

        for idx, task_name in enumerate(task_names):
            read_offset = read_offsets_list[idx]
            if read_offset is not None:
                read_offset = int(read_offset)

            task_ack_offset = task_offsets_list[idx]
            if task_ack_offset is not None:
                task_ack_offset = int(task_ack_offset)

            pending_read = (task_offset - read_offset) if read_offset is not None else None
            pending_consume = (task_offset - task_ack_offset) if task_ack_offset is not None else None

            results.append({
                "task_name": task_name,
                "task_offset": task_offset,
                "read_offset": read_offset,
                "task_ack_offset": task_ack_offset,
                "pending_read": pending_read,
                "pending_consume": pending_consume
            })

        if return_single:
            return results[0] if results else None
        return results

    async def _get_queue_positions_async(self, event_id: str, queue: str, task_names: list, return_single: bool):
        results = []

        prefixed_queue = f"{self.redis_prefix}:QUEUE:{queue}"

        try:
            stream_data = await self.async_binary_redis.xrange(prefixed_queue, min=event_id, max=event_id, count=1)

            if not stream_data:
                for task_name in task_names:
                    results.append({
                        "task_name": task_name,
                        "error": "Task not found in stream"
                    })
                if return_single:
                    return results[0] if results else None
                return results

            message_id, message_data = stream_data[0]

            task_offset = None
            for key, value in message_data.items():
                if isinstance(key, bytes):
                    key = key.decode('utf-8')
                if key == 'offset':
                    if isinstance(value, bytes):
                        task_offset = int(value.decode('utf-8'))
                    else:
                        task_offset = int(value)
                    break

            if task_offset is None:
                for task_name in task_names:
                    results.append({
                        "task_name": task_name,
                        "error": "Offset not found in task data"
                    })
                if return_single:
                    return results[0] if results else None
                return results

        except Exception as e:
            for task_name in task_names:
                results.append({
                    "task_name": task_name,
                    "error": f"Failed to read from stream: {str(e)}"
                })
            if return_single:
                return results[0] if results else None
            return results

        read_offsets_key = f"{self.redis_prefix}:READ_OFFSETS"
        task_offsets_key = f"{self.redis_prefix}:TASK_OFFSETS"

        base_queue = queue.split(':')[0] if ':' in queue else queue

        offset_keys = [f"{base_queue}:{task_name}" for task_name in task_names]

        try:
            pipeline = self.async_redis.pipeline()
            pipeline.hmget(read_offsets_key, offset_keys)
            pipeline.hmget(task_offsets_key, offset_keys)
            read_offsets_list, task_offsets_list = await pipeline.execute()
        except Exception as e:
            for task_name in task_names:
                results.append({
                    "task_name": task_name,
                    "error": f"Failed to read offsets: {str(e)}"
                })
            if return_single:
                return results[0] if results else None
            return results

        for idx, task_name in enumerate(task_names):
            read_offset = read_offsets_list[idx]
            if read_offset is not None:
                read_offset = int(read_offset)

            task_ack_offset = task_offsets_list[idx]
            if task_ack_offset is not None:
                task_ack_offset = int(task_ack_offset)

            pending_read = (task_offset - read_offset) if read_offset is not None else None
            pending_consume = (task_offset - task_ack_offset) if task_ack_offset is not None else None

            results.append({
                "task_name": task_name,
                "task_offset": task_offset,
                "read_offset": read_offset,
                "task_ack_offset": task_ack_offset,
                "pending_read": pending_read,
                "pending_consume": pending_consume
            })

        if return_single:
            return results[0] if results else None
        return results

    def _build_task_key(self, task_name: str, queue: str, event_id: str):
        prefixed_queue = f"{self.redis_prefix}:QUEUE:{queue}"
        group_name = f"{prefixed_queue}:{task_name}"
        status_key = f"{event_id}:{group_name}"
        full_key = f"{self.redis_prefix}:TASK:{status_key}"
        return group_name, full_key

    @staticmethod
    def _decode_bytes(value):
        if value and isinstance(value, bytes):
            return value.decode('utf-8')
        return value

    @staticmethod
    def _is_task_completed(status):
        return status in [TaskStatus.COMPLETED.value, TaskStatus.SUCCESS.value]

    @staticmethod
    def _is_task_failed(status):
        return status in [TaskStatus.ERROR.value, TaskStatus.FAILED.value, "ERROR", "FAILED", "error", "failed"]

    def _get_results_sync(self, event_id: str, queue: str, task_names: list,
                         delete: bool, delayed_deletion_ex: int, wait: bool,
                         timeout: int, poll_interval: float, return_single: bool):
        results = []

        for task_name in task_names:
            try:
                _, full_key = self._build_task_key(task_name, queue, event_id)

                task_info = self._get_result_sync(full_key, event_id, delete, delayed_deletion_ex,
                                                  wait, timeout, poll_interval)

                if not task_info:
                    results.append({
                        "task_name": task_name,
                        "status": None,
                        "result": None
                    })
                else:
                    task_info["task_name"] = task_name
                    results.append(task_info)

            except Exception as e:
                results.append({
                    "task_name": task_name,
                    "status": "ERROR",
                    "result": None,
                    "error_msg": str(e)
                })

        if return_single:
            return results[0] if results else None
        return results

    async def _get_results_async(self, event_id: str, queue: str, task_names: list,
                                 delete: bool, delayed_deletion_ex: int, wait: bool,
                                 timeout: int, poll_interval: float, return_single: bool):
        results = []

        for task_name in task_names:
            try:
                _, full_key = self._build_task_key(task_name, queue, event_id)

                task_info = await self._get_result_async(full_key, event_id, delete, delayed_deletion_ex,
                                                         wait, timeout, poll_interval)

                if not task_info:
                    results.append({
                        "task_name": task_name,
                        "status": TaskStatus.PENDING.value,
                        "result": None
                    })
                else:
                    task_info["task_name"] = task_name
                    results.append(task_info)

            except Exception as e:
                results.append({
                    "task_name": task_name,
                    "status": "ERROR",
                    "result": None,
                    "error_msg": str(e)
                })

        if return_single:
            return results[0] if results else None
        return results

    def _get_result_sync(self, full_key: str, event_id: str, delete: bool, delayed_deletion_ex: int,
                         wait: bool = False, timeout: int = 300, poll_interval: float = 0.5):
        from ..exceptions import TaskTimeoutError, TaskExecutionError, TaskNotFoundError

        client = self.binary_redis
        start_time = time.time()

        while True:
            task_data = client.hgetall(full_key)

            if not task_data:
                if wait:
                    raise TaskNotFoundError(f"Task {event_id} not found")
                return None

            decoded_data = {}
            for key, value in task_data.items():
                if isinstance(key, bytes):
                    key = key.decode('utf-8')

                if key.startswith('__'):
                    continue

                if isinstance(value, bytes):
                    if key == 'result':
                        try:
                            decoded_data[key] = loads_str(value)
                        except Exception:
                            decoded_data[key] = value
                    else:
                        try:
                            decoded_data[key] = value.decode('utf-8')
                        except Exception:
                            decoded_data[key] = value
                else:
                    decoded_data[key] = value

            if not wait:
                if delayed_deletion_ex is not None:
                    client.expire(full_key, delayed_deletion_ex)
                elif delete:
                    if self.task_center and self.task_center.is_enabled:
                        client.hset(full_key, "__pending_delete", "1")
                    else:
                        client.delete(full_key)
                return decoded_data

            status = decoded_data.get('status')

            if self._is_task_completed(status):
                if delayed_deletion_ex is not None:
                    client.expire(full_key, delayed_deletion_ex)
                elif delete:
                    if self.task_center and self.task_center.is_enabled:
                        client.hset(full_key, "__pending_delete", "1")
                    else:
                        client.delete(full_key)
                return decoded_data

            elif self._is_task_failed(status):
                error_msg = decoded_data.get('error_msg', 'Task execution failed')
                raise TaskExecutionError(event_id, error_msg)

            if time.time() - start_time > timeout:
                raise TaskTimeoutError(f"Task {event_id} timed out after {timeout} seconds")

            time.sleep(poll_interval)

    async def _get_result_async(self, full_key: str, event_id: str, delete: bool, delayed_deletion_ex: int,
                                wait: bool = False, timeout: int = 300, poll_interval: float = 0.5):

        client = self.async_binary_redis
        start_time = time.time()

        while True:
            task_data = await client.hgetall(full_key)

            if not task_data:
                if wait:
                    raise TaskNotFoundError(f"Task {event_id} not found")
                return None

            decoded_data = {}
            for key, value in task_data.items():
                if isinstance(key, bytes):
                    key = key.decode('utf-8')

                if key.startswith('__'):
                    continue

                if isinstance(value, bytes):
                    if key == 'result':
                        try:
                            decoded_data[key] = loads_str(value)
                        except Exception:
                            decoded_data[key] = value
                    else:
                        try:
                            decoded_data[key] = value.decode('utf-8')
                        except Exception:
                            decoded_data[key] = value
                else:
                    decoded_data[key] = value

            if not wait:
                if delayed_deletion_ex is not None:
                    await client.expire(full_key, delayed_deletion_ex)
                elif delete:
                    if self.task_center and self.task_center.is_enabled:
                        await client.hset(full_key, "__pending_delete", "1")
                    else:
                        await client.delete(full_key)
                return decoded_data

            status = decoded_data.get('status')

            if self._is_task_completed(status):
                if delayed_deletion_ex is not None:
                    await client.expire(full_key, delayed_deletion_ex)
                elif delete:
                    if self.task_center and self.task_center.is_enabled:
                        await client.hset(full_key, "__pending_delete", "1")
                    else:
                        await client.delete(full_key)
                return decoded_data

            elif self._is_task_failed(status):
                error_msg = decoded_data.get('error_msg', 'Task execution failed')
                raise TaskExecutionError(event_id, error_msg)

            if time.time() - start_time > timeout:
                raise TaskTimeoutError(f"Task {event_id} timed out after {timeout} seconds")

            await asyncio.sleep(poll_interval)

    def register_router(self, router, prefix: str = None):
        from ..task.router import TaskRouter
        
        if not isinstance(router, TaskRouter):
            raise TypeError("router must be a TaskRouter instance")
        
        for task_name, task_config in router.get_tasks().items():
            if prefix:
                if task_config.get('name'):
                    task_config['name'] = f"{prefix}.{task_config['name']}"
                task_name = f"{prefix}.{task_name}"
            
            func = task_config.pop('func')
            name = task_config.pop('name', task_name)
            queue = task_config.pop('queue', None)
            
            task = self._task_from_fun(func, name, None, queue, **task_config)
            logger.debug(f"Registered task: {name} (queue: {queue or self.redis_prefix})")
        
        return self

    def _mount_module(self):
        for module in self.include:
            module = importlib.import_module(module)
            for attr_name in dir(module):
                obj = getattr(module, attr_name)
                if hasattr(obj, "app"):
                    self._tasks.update(getattr(obj, "app")._tasks)

    def _validate_tasks_for_executor(self, execute_type: str, queues: List[str]):
        if execute_type in ["asyncio", "multi_asyncio"]:
            return  
        
        incompatible_tasks = []
        for task_name, task in self._tasks.items():
            if task.queue not in queues:
                continue
                
            if asyncio.iscoroutinefunction(task.run):
                incompatible_tasks.append({
                    'name': task_name,
                    'queue': task.queue,
                    'type': 'async'
                })
        
        if incompatible_tasks:
            error_msg = f"\n错误：{execute_type} 执行器不能处理异步任务！\n"
            error_msg += "发现以下异步任务：\n"
            for task in incompatible_tasks:
                error_msg += f"  - {task['name']} (队列: {task['queue']})\n"
            error_msg += f"\n解决方案：\n"
            error_msg += f"1. 使用 asyncio 或 process 执行器\n"
            error_msg += f"2. 或者将这些任务改为同步函数（去掉 async/await）\n"
            error_msg += f"3. 或者将这些任务的队列从监听列表中移除\n"
            raise ValueError(error_msg)



    def _create_executor(self, concurrency: int):
        orchestrator = ProcessOrchestrator(self, concurrency)

        self._current_executor = orchestrator

        def signal_handler(signum, frame):
            logger.debug(f"Main process received signal {signum}, initiating shutdown...")
            self._should_exit = True
            orchestrator.shutdown_event.set()
            raise KeyboardInterrupt()

        signal.signal(signal.SIGINT, signal_handler)
        signal.signal(signal.SIGTERM, signal_handler)

        return orchestrator

    async def _cleanup_worker_v3(self, heartbeat_managers: list, executor, worker_ids: list):
        logger.debug("Shutting down workers...")


        if heartbeat_managers:
            logger.debug(f"Stopping {len(heartbeat_managers)} heartbeat tasks...")
            for i, heartbeat in enumerate(heartbeat_managers):
                try:
                    worker_id = worker_ids[i][0] if i < len(worker_ids) else f"worker_{i}"
                    logger.debug(f"Stopping heartbeat task for {worker_id}...")
                    await heartbeat.stop()
                except Exception as e:
                    logger.error(f"Error stopping heartbeat #{i}: {e}", exc_info=True)

        if executor:
            try:
                logger.debug("Shutting down executor...")
                executor.shutdown()
            except Exception as e:
                logger.error(f"Error shutting down executor: {e}", exc_info=True)


        logger.debug(f"All {len(worker_ids)} workers shutdown complete")

    async def _start(self, tasks: List[str], concurrency: int = 1, prefetch_multiplier: int = 1):
        heartbeat_managers = []
        executor = None
        worker_ids = []

        queues = self._get_queues_from_tasks(tasks)
        logger.debug(f"Tasks {tasks} -> Queues {queues}")

        try:
            from jettask.worker.heartbeat import HeartbeatManager

            for i in range(concurrency):
                heartbeat = await HeartbeatManager.create_and_start(
                    queue_registry=self.registry,  
                    worker_manager=self.worker_state,  
                    async_redis_client=self.async_redis,
                    redis_prefix=self.redis_prefix,
                    interval=self.heartbeat_interval,
                    heartbeat_timeout=self.heartbeat_timeout,
                    worker_state=self.worker_state
                )
                worker_ids.append((heartbeat.worker_id, heartbeat.worker_key))
                heartbeat_managers.append(heartbeat)

            executor = self._create_executor(concurrency)

            HeartbeatManager.start_global_timeout_scanner(
                scan_interval=self.scanner_interval,
                worker_manager=self.worker_state
            )
         


            loop = asyncio.get_event_loop()
            await loop.run_in_executor(
                None,  
                executor.start,
                queues,
                tasks,
                prefetch_multiplier,
                worker_ids
            )
        

        except KeyboardInterrupt:
            logger.debug("Worker interrupted by keyboard")
        except Exception as e:
            logger.error(f"Error in worker main loop: {e}", exc_info=True)
        finally:
            logger.debug("Stopping global worker timeout scanner...")
            await HeartbeatManager.stop_global_timeout_scanner()

            await self._cleanup_worker_v3(heartbeat_managers, executor, worker_ids)

    def start(
        self,
        tasks: List[str],
        concurrency: int = 1,
        prefetch_multiplier: int = 1,
    ):

        if not tasks:
            raise ValueError(
                "必须指定 tasks 参数！\n"
                "示例: app.start(tasks=['task1', 'task2'], concurrency=4)"
            )

        self._worker_started = True

        if self.task_center and self.task_center.is_enabled and not self._task_center_config:
            self._load_config_from_task_center()

        logger.debug("正在注册待注册的限流配置...")
        self._apply_pending_rate_limits()

        self._setup_cleanup_handlers()


        asyncio.run(self._start(
            tasks=tasks,
            concurrency=concurrency,
            prefetch_multiplier=prefetch_multiplier,
        ))


    def get_task_info(self, event_id: str, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        key = f"{self.redis_prefix}:TASK:{event_id}"
        if asyncio:
            return client.hgetall(key)
        else:
            return client.hgetall(key)
    
    def get_task_status(self, event_id: str, asyncio: bool = False):
        if asyncio:
            return self._get_task_status_simple_async(event_id)
        else:
            client = self.redis
            key = f"{self.redis_prefix}:TASK:{event_id}"
            return client.hget(key, "status")

    async def _get_task_status_simple_async(self, event_id: str):
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return await self.async_redis.hget(key, "status")

    def set_task_status(self, event_id: str, status: str, asyncio: bool = False):
        if asyncio:
            return self._set_task_status_async(event_id, status)
        else:
            client = self.redis
            key = f"{self.redis_prefix}:TASK:{event_id}"
            return client.hset(key, "status", status)
    
    async def _set_task_status_async(self, event_id: str, status: str):
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return await self.async_redis.hset(key, "status", status)

    def set_task_status_by_batch(self, mapping: dict, asyncio: bool = False):
        if asyncio:
            return self._set_task_status_by_batch_async(mapping)
        else:
            pipeline = self.redis.pipeline()
            for event_id, status in mapping.items():
                key = f"{self.redis_prefix}:TASK:{event_id}"
                pipeline.hset(key, "status", status)
            return pipeline.execute()
    
    async def _set_task_status_by_batch_async(self, mapping: dict):
        pipeline = self.async_redis.pipeline()
        for event_id, status in mapping.items():
            key = f"{self.redis_prefix}:TASK:{event_id}"
            pipeline.hset(key, "status", status)
        return await pipeline.execute()

    def del_task_status(self, event_id: str, asyncio: bool = False):
        client = self.get_redis_client(asyncio)
        key = f"{self.redis_prefix}:TASK:{event_id}"
        return client.delete(key)

    def get_redis_client(self, asyncio: bool = False):
        return self.async_redis if asyncio else self.redis

    async def get_and_delayed_deletion(self, key: str, ex: int):
        result = await self.async_redis.hget(key, "result")
        await self.async_redis.expire(key, ex)
        return result

    
    async def _ensure_scheduler_initialized(self, db_url: str = None):
        if not self.scheduler:
            logger.debug("Auto-initializing scheduler...")
            if not db_url:
                db_url = self.pg_url or os.environ.get('JETTASK_PG_URL')
            if not db_url:
                raise ValueError(
                    "Database URL not provided. Please provide pg_url when initializing Jettask, "
                    "or set JETTASK_PG_URL environment variable\n"
                    "Example: app = Jettask(redis_url='...', pg_url='postgresql://...')\n"
                    "Or: export JETTASK_PG_URL='postgresql://user:password@localhost:5432/jettask'"
                )

            self._scheduler_db_url = db_url

            scheduler_config = self.scheduler_config.copy()
            scheduler_config.setdefault('scan_interval', 1.0)  
            scheduler_config.setdefault('batch_size', 100)
            scheduler_config.setdefault('lookahead_seconds', 3600)  

            self.scheduler = TaskScheduler(
                app=self,
                db_url=db_url,
                **scheduler_config
            )

            logger.debug("Scheduler initialized")
    
    async def start_scheduler(self):
        await self._ensure_scheduler_initialized()

        try:
            await self.scheduler.start(wait=True)
        except Exception as e:
            logger.error(f"Scheduler error: {e}", exc_info=True)
            raise
    
    async def register_schedules(self, schedules):
        from ..scheduler.definition import Schedule
        from ..db.models import ScheduledTask, TaskType

        await self._ensure_scheduler_initialized()

        if isinstance(schedules, Schedule):
            schedules = [schedules]

        if not schedules:
            return 0

        namespace = 'default'
        if self.task_center and hasattr(self.task_center, 'namespace_name'):
            namespace = self.task_center.namespace_name
        elif self.redis_prefix and self.redis_prefix != 'jettask':
            namespace = self.redis_prefix

        tasks = []
        for schedule in schedules:
            if not isinstance(schedule, Schedule):
                raise ValueError(f"Expected Schedule, got {type(schedule)}")

            data = schedule.to_dict()
            task = ScheduledTask(
                scheduler_id=data['scheduler_id'],
                task_type=data['task_type'],  
                queue_name=data['queue'],
                namespace=namespace,
                task_args=data['task_args'],
                task_kwargs=data['task_kwargs'],
                interval_seconds=data.get('interval_seconds'),
                cron_expression=data.get('cron_expression'),
                next_run_time=data.get('next_run_time'),
                enabled=data['enabled'],
                priority=data['priority'],
                timeout=data['timeout'],
                max_retries=data['max_retries'],
                retry_delay=data['retry_delay'],
                description=data['description'],
                tags=data['tags'],
                metadata=data['metadata']
            )
            task._skip_if_exists = schedule.skip_if_exists
            tasks.append(task)

        from ..db.connector import get_pg_engine_and_factory

        _, session_factory = get_pg_engine_and_factory(
            dsn=self._scheduler_db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )
        session = session_factory()
        try:
            registered_count = 0
            for task in tasks:
                skip_if_exists = getattr(task, '_skip_if_exists', True)

                existing_task = await ScheduledTask.get_by_scheduler_id(session, task.scheduler_id)

                if existing_task:
                    if not skip_if_exists:
                        task.id = existing_task.id
                        await ScheduledTask.update_task(session, task)
                        await session.commit()
                        logger.debug(f"已更新定时任务: {task.scheduler_id} -> {task.queue_name}")
                    else:
                        logger.debug(f"定时任务已存在: {task.scheduler_id}")
                else:
                    await ScheduledTask.create(session, task)
                    await session.commit()
                    registered_count += 1
                    logger.debug(f"已注册定时任务: {task.scheduler_id} -> {task.queue_name}")

            return registered_count
        finally:
            await session.close()

    async def list_schedules(self, **filters):
        await self._ensure_scheduler_initialized()

        from ..db.connector import get_pg_engine_and_factory
        from ..db.models import ScheduledTask

        _, session_factory = get_pg_engine_and_factory(
            dsn=self._scheduler_db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )
        session = session_factory()
        try:
            return await ScheduledTask.list_tasks(session, **filters)
        finally:
            await session.close()

    async def remove_schedule(self, scheduler_id: str) -> bool:
        await self._ensure_scheduler_initialized()

        from ..db.connector import get_pg_engine_and_factory
        from ..db.models import ScheduledTask

        _, session_factory = get_pg_engine_and_factory(
            dsn=self._scheduler_db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )
        session = session_factory()
        try:
            task = await ScheduledTask.get_by_scheduler_id(session, scheduler_id)
            if not task:
                return False

            await ScheduledTask.delete_by_id(session, task.id)
            await session.commit()
            return True
        finally:
            await session.close()

    async def pause_schedule(self, scheduler_id: str) -> bool:
        await self._ensure_scheduler_initialized()

        from ..db.connector import get_pg_engine_and_factory
        from ..db.models import ScheduledTask

        _, session_factory = get_pg_engine_and_factory(
            dsn=self._scheduler_db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )
        session = session_factory()
        try:
            task = await ScheduledTask.get_by_scheduler_id(session, scheduler_id)
            if not task:
                return False

            task.enabled = False
            await ScheduledTask.update_task(session, task)
            await session.commit()
            return True
        finally:
            await session.close()

    async def resume_schedule(self, scheduler_id: str) -> bool:
        await self._ensure_scheduler_initialized()

        from ..db.connector import get_pg_engine_and_factory
        from ..db.models import ScheduledTask

        _, session_factory = get_pg_engine_and_factory(
            dsn=self._scheduler_db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )
        session = session_factory()
        try:
            task = await ScheduledTask.get_by_scheduler_id(session, scheduler_id)
            if not task:
                return False

            task.enabled = True
            await ScheduledTask.update_task(session, task)
            await session.commit()
            return True
        finally:
            await session.close()




"""
PostgreSQL 时间轴服务

从 PostgreSQL 数据库获取任务时间分布数据
"""
import logging
from datetime import datetime, timedelta, timezone
from typing import Dict, List, Any, Optional
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy import text
from sqlalchemy.dialects import postgresql

logger = logging.getLogger(__name__)


class TimelinePgService:
    """PostgreSQL 时间轴服务"""

    def __init__(self):
        pass

    @staticmethod
    def parse_iso_datetime(time_str: str) -> datetime:
        if time_str.endswith('Z'):
            dt = datetime.fromisoformat(time_str.replace('Z', '+00:00'))
        else:
            dt = datetime.fromisoformat(time_str)

        if dt.tzinfo is None:
            dt = dt.replace(tzinfo=timezone.utc)
        elif dt.tzinfo != timezone.utc:
            dt = dt.astimezone(timezone.utc)

        return dt

    @staticmethod
    def parse_interval(interval: str) -> int:
        if interval.endswith('m'):
            return int(interval[:-1])
        elif interval.endswith('h'):
            return int(interval[:-1]) * 60
        elif interval.endswith('s'):
            return int(interval[:-1]) // 60 if int(interval[:-1]) >= 60 else 1
        else:
            return 5  

    @staticmethod
    def calculate_auto_interval(duration_seconds: float) -> tuple:
        if duration_seconds <= 300:  
            return 0.5, 'millisecond', '500ms'
        elif duration_seconds <= 900:  
            return 1, 'second', '1s'
        elif duration_seconds <= 1800:  
            return 2, 'second', '2s'
        elif duration_seconds <= 3600:  
            return 30, 'second', '30s'
        elif duration_seconds <= 10800:  
            return 300, 'minute', '5m'
        elif duration_seconds <= 21600:  
            return 600, 'minute', '10m'
        elif duration_seconds <= 43200:  
            return 1800, 'minute', '30m'
        elif duration_seconds <= 86400:  
            return 3600, 'hour', '1h'
        elif duration_seconds <= 172800:  
            return 7200, 'hour', '2h'
        elif duration_seconds <= 604800:  
            return 21600, 'hour', '6h'
        else:  
            return 86400, 'hour', '24h'

    @staticmethod
    def align_to_interval(dt: datetime, interval_seconds: float) -> datetime:
        if interval_seconds >= 3600:  
            dt = dt.replace(minute=0, second=0, microsecond=0)
            interval_hours = int(interval_seconds // 3600)
            aligned_hour = (dt.hour // interval_hours) * interval_hours
            return dt.replace(hour=aligned_hour)
        elif interval_seconds >= 60:  
            dt = dt.replace(second=0, microsecond=0)
            interval_minutes = int(interval_seconds // 60)
            total_minutes = dt.hour * 60 + dt.minute
            aligned_total_minutes = (total_minutes // interval_minutes) * interval_minutes
            aligned_hour = aligned_total_minutes // 60
            aligned_minute = aligned_total_minutes % 60
            return dt.replace(hour=aligned_hour, minute=aligned_minute)
        elif interval_seconds >= 1:  
            dt = dt.replace(microsecond=0)
            aligned_second = int(dt.second // interval_seconds) * int(interval_seconds)
            return dt.replace(second=aligned_second)
        else:  
            total_ms = dt.microsecond / 1000  
            interval_ms = interval_seconds * 1000
            aligned_ms = int(total_ms // interval_ms) * interval_ms
            aligned_microsecond = int(aligned_ms * 1000)
            return dt.replace(microsecond=aligned_microsecond)

    async def get_single_queue_timeline(
        self,
        session: AsyncSession,
        queue_name: str,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        interval: str = "5m"
    ) -> Dict[str, Any]:
        if not end_time:
            end_dt = datetime.now(timezone.utc)
        else:
            end_dt = self.parse_iso_datetime(end_time)

        if not start_time:
            start_dt = end_dt - timedelta(hours=1)
        else:
            start_dt = self.parse_iso_datetime(start_time)

        interval_minutes = self.parse_interval(interval)

        try:
            query = text(f"""
            SELECT
                DATE_TRUNC('minute', created_at) -
                INTERVAL '{interval_minutes} minutes' * (EXTRACT(MINUTE FROM created_at)::int % {interval_minutes}) as time_bucket,
                COUNT(*) as count,
                SUM(CASE WHEN status = 'completed' THEN 1 ELSE 0 END) as completed_count,
                SUM(CASE WHEN status = 'failed' THEN 1 ELSE 0 END) as failed_count,
                AVG(CASE WHEN status = 'completed' AND processing_time IS NOT NULL
                    THEN processing_time ELSE NULL END) as avg_processing_time
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)

            result = await session.execute(query, {
                'queue_name': queue_name,
                'start_dt': start_dt,
                'end_dt': end_dt
            })
            rows = result.mappings().all()

            timeline = []
            for row in rows:
                timeline.append({
                    "time": row['time_bucket'].isoformat(),
                    "count": row['count'],
                    "completed_count": row['completed_count'],
                    "failed_count": row['failed_count'],
                    "avg_processing_time": float(row['avg_processing_time']) if row['avg_processing_time'] else 0
                })

            filled_timeline = []
            current_time = start_dt
            timeline_dict = {item['time']: item for item in timeline}

            while current_time < end_dt:
                time_key = current_time.isoformat()
                if time_key in timeline_dict:
                    filled_timeline.append(timeline_dict[time_key])
                else:
                    filled_timeline.append({
                        "time": time_key,
                        "count": 0,
                        "completed_count": 0,
                        "failed_count": 0,
                        "avg_processing_time": 0
                    })
                current_time += timedelta(minutes=interval_minutes)

            return {
                "timeline": filled_timeline,
                "interval": interval,
                "start_time": start_dt.isoformat(),
                "end_time": end_dt.isoformat()
            }

        except Exception as e:
            logger.error(f"Error fetching timeline from PostgreSQL: {e}")
            return {
                "timeline": [],
                "interval": interval,
                "start_time": start_dt.isoformat(),
                "end_time": end_dt.isoformat(),
                "error": str(e)
            }

    def _build_time_bucket_query(
        self,
        interval_type: str,
        interval_seconds: float,
        interval_minutes: float
    ) -> text:
        if interval_type == 'millisecond':
            return text(f"""
            SELECT
                DATE_TRUNC('second', created_at) +
                INTERVAL '{interval_seconds} seconds' * FLOOR(EXTRACT(MILLISECONDS FROM created_at) / ({interval_seconds} * 1000)) as time_bucket,
                COUNT(*) as count
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)
        elif interval_type == 'second':
            return text(f"""
            SELECT
                DATE_TRUNC('minute', created_at) +
                INTERVAL '{interval_seconds} seconds' * FLOOR(EXTRACT(SECOND FROM created_at) / {interval_seconds}) as time_bucket,
                COUNT(*) as count
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)
        elif interval_type == 'minute' and interval_minutes < 60:
            return text(f"""
            SELECT
                DATE_TRUNC('hour', created_at) +
                INTERVAL '{interval_minutes} minutes' * FLOOR(EXTRACT(MINUTE FROM created_at) / {interval_minutes}) as time_bucket,
                COUNT(*) as count
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)
        elif interval_minutes == 60:
            return text("""
            SELECT
                DATE_TRUNC('hour', created_at) as time_bucket,
                COUNT(*) as count
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)
        else:
            interval_hours = int(interval_minutes // 60)
            return text(f"""
            SELECT
                DATE_TRUNC('day', created_at) +
                INTERVAL '{interval_hours} hours' * FLOOR(EXTRACT(HOUR FROM created_at) / {interval_hours}) as time_bucket,
                COUNT(*) as count
            FROM tasks
            WHERE queue_name = :queue_name
                AND created_at >= :start_dt
                AND created_at < :end_dt
            GROUP BY time_bucket
            ORDER BY time_bucket
            """)

    async def get_multiple_queues_timeline(
        self,
        session_factory,
        queues: str,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None
    ) -> Dict[str, Any]:
        if not queues or queues.strip() == "":
            end_dt = datetime.now(timezone.utc) if not end_time else self.parse_iso_datetime(end_time)
            start_dt = (end_dt - timedelta(hours=1)) if not start_time else self.parse_iso_datetime(start_time)

            return {
                "queues": [],
                "start_time": start_dt.isoformat(),
                "end_time": end_dt.isoformat(),
                "interval": "5m",
                "message": "No queues selected"
            }

        queue_list = [q.strip() for q in queues.split(',') if q.strip()][:10]  

        if not end_time:
            end_dt = datetime.now(timezone.utc)
        else:
            end_dt = self.parse_iso_datetime(end_time)

        if not start_time:
            start_dt = end_dt - timedelta(hours=1)
        else:
            start_dt = self.parse_iso_datetime(start_time)

        logger.info(f'start_dt={start_dt} end_dt={end_dt}')

        duration = (end_dt - start_dt).total_seconds()
        interval_seconds, interval_type, interval = self.calculate_auto_interval(duration)
        interval_minutes = interval_seconds / 60

        logger.info(f"Time range: {duration}s, using interval: {interval} -> {interval_seconds} seconds, type: {interval_type}")

        result = []

        for queue_name in queue_list:
            try:
                async with session_factory() as session:
                    query = self._build_time_bucket_query(interval_type, interval_seconds, interval_minutes)

                    params = {
                        'queue_name': queue_name,
                        'start_dt': start_dt,
                        'end_dt': end_dt
                    }

                    result_obj = await session.execute(query, params)
                    rows = result_obj.mappings().all()
                    logger.info(f'rows={rows}')

                    timeline = []
                    for row in rows:
                        timeline.append({
                            "time": row['time_bucket'].isoformat(),
                            "count": row['count']
                        })

                    timeline_data = []
                    for item in timeline:
                        dt = datetime.fromisoformat(item['time'])
                        timeline_data.append((dt, item['count']))

                    timeline_data.sort(key=lambda x: x[0])

                    filled_timeline = []
                    current_time = self.align_to_interval(start_dt, interval_seconds)
                    timeline_index = 0

                    while current_time < end_dt:
                        tolerance = timedelta(seconds=interval_seconds/2)
                        found = False

                        while timeline_index < len(timeline_data):
                            data_time, count = timeline_data[timeline_index]

                            time_diff = abs((data_time - current_time).total_seconds())

                            if time_diff < interval_seconds / 2:
                                filled_timeline.append({
                                    "time": current_time.isoformat(),
                                    "count": count
                                })
                                found = True
                                timeline_index += 1
                                break
                            elif data_time > current_time + tolerance:
                                break
                            else:
                                timeline_index += 1

                        if not found:
                            filled_timeline.append({
                                "time": current_time.isoformat(),
                                "count": 0
                            })

                        current_time += timedelta(seconds=interval_seconds)

                    result.append({
                        "queue": queue_name,
                        "timeline": {
                            "timeline": filled_timeline,
                            "interval": interval
                        }
                    })

            except Exception as e:
                logger.error(f"Error fetching timeline for queue {queue_name}: {e}")
                result.append({
                    "queue": queue_name,
                    "timeline": {
                        "timeline": [],
                        "interval": interval,
                        "error": str(e)
                    }
                })

        return {
            "queues": result,
            "start_time": start_dt.isoformat(),
            "end_time": end_dt.isoformat(),
            "interval": interval
        }



"""
定时任务调度模块

本模块提供：
- TaskScheduler - 定时任务调度器，负责周期性扫描和触发任务
"""
import asyncio
import logging
from typing import Optional, List
from datetime import datetime, timezone
import time
from sqlalchemy.ext.asyncio import AsyncSession, async_sessionmaker

from jettask.core.message import TaskMessage
from jettask.db.models import ScheduledTask, TaskType
from jettask.db.connector import get_pg_engine_and_factory, _get_safe_pg_dsn

logger = logging.getLogger('app')




class TaskScheduler:
    """
    定时任务调度器

    设计理念：
    1. 直接从数据库读取即将到期的任务（避免 Redis 中间层）
    2. 提前1小时获取任务，利用延迟任务功能（TaskMessage 的 delay 参数）
    3. 发送后立即更新数据库状态，防止重复发送
    4. 减少数据库查询频率，提高系统可维护性

    工作流程：
    1. 定期扫描数据库（默认30秒一次）
    2. 查找未来1小时内需要执行的任务
    3. 使用 TaskMessage + delay 发送到队列
    4. 更新任务状态（周期性任务计算下次执行时间，一次性任务禁用）
    """

    def __init__(
        self,
        app,
        db_url: str,
        scan_interval: float = 3.0,
        batch_size: int = 100,
        lookahead_seconds: int = 3600,  
        **kwargs  
    ):
        self.app = app
        self.db_url = db_url

        self.scan_interval = scan_interval
        self.batch_size = batch_size
        self.lookahead_seconds = lookahead_seconds

        _, self._session_factory = get_pg_engine_and_factory(
            dsn=db_url,
            pool_size=10,
            max_overflow=20,
            pool_recycle=3600,
            echo=False
        )

        self._running = False
        self._task: Optional[asyncio.Task] = None
        self._stop_event = asyncio.Event()

        logger.info(
            f"TaskScheduler initialized: "
            f"scan_interval={scan_interval}s, "
            f"batch_size={batch_size}, "
            f"lookahead={lookahead_seconds}s, "
            f"db_url={_get_safe_pg_dsn(db_url)}"
        )

    async def start(self, wait: bool = False):
        if self._running:
            logger.warning("Scheduler already running")
            return

        self._running = True
        self._stop_event.clear()

        self._task = asyncio.create_task(self._run_loop())

        logger.info("TaskScheduler started")

        if wait:
            try:
                await self._task
            except asyncio.CancelledError:
                logger.info("Scheduler was cancelled")
            except Exception as e:
                logger.error(f"Scheduler error: {e}", exc_info=True)
                raise

    async def stop(self):
        if not self._running:
            logger.warning("Scheduler is not running")
            return

        logger.info("Stopping TaskScheduler...")

        self._running = False
        self._stop_event.set()

        if self._task and not self._task.done():
            try:
                await asyncio.wait_for(self._task, timeout=10.0)
            except asyncio.TimeoutError:
                logger.warning("Scheduler task did not finish in time, cancelling...")
                self._task.cancel()
                try:
                    await self._task
                except asyncio.CancelledError:
                    pass
            except Exception as e:
                logger.error(f"Error while stopping scheduler: {e}", exc_info=True)

        logger.info("TaskScheduler stopped")

    async def _run_loop(self):
        logger.info("Scheduler loop started")

        try:
            while self._running:
                try:
                    await self._schedule_once()
                    print(f'{self.scan_interval=}')
                    try:
                        await asyncio.wait_for(
                            self._stop_event.wait(),
                            timeout=self.scan_interval
                        )
                        break
                    except asyncio.TimeoutError:
                        pass

                except Exception as e:
                    logger.error(f"Error in scheduler loop: {e}", exc_info=True)
                    await asyncio.sleep(5)

        except asyncio.CancelledError:
            logger.info("Scheduler loop cancelled")

        finally:
            logger.info("Scheduler loop exited")

    async def _schedule_once(self):
        start_time = time.time()
        logger.debug("开始调度检查...")  

        session = self._session_factory()
        try:
            async with session.begin():
                tasks = await ScheduledTask.get_ready_tasks(
                    session,
                    batch_size=self.batch_size,
                    lookahead_seconds=self.lookahead_seconds
                )

                if not tasks:
                    logger.debug("No tasks ready to schedule")
                    return 0

                logger.info(f"Found {len(tasks)} tasks ready to schedule (within {self.lookahead_seconds}s)")

                scheduled_count = 0
                failed_tasks = []

                for task in tasks:
                    try:
                        success = await self._send_task_with_delay(task)

                        if success:
                            scheduled_count += 1
                        else:
                            failed_tasks.append(task)

                    except Exception as e:
                        logger.error(
                            f"Error scheduling task {task.scheduler_id}: {e}",
                            exc_info=True
                        )
                        failed_tasks.append(task)

                await self._update_tasks_after_scheduling(session, tasks)

        finally:
            await session.close()

        duration = time.time() - start_time
        logger.info(
            f"Schedule cycle completed: "
            f"scheduled={scheduled_count}, "
            f"failed={len(failed_tasks)}, "
            f"duration={duration:.2f}s"
        )

        if failed_tasks:
            failed_ids = [t.scheduler_id for t in failed_tasks]
            logger.warning(f"Failed to schedule tasks: {failed_ids}")

        return scheduled_count

    async def _send_task_with_delay(self, task: ScheduledTask) -> bool:

        now = datetime.now(timezone.utc)
        delay = None

        if task.next_run_time:
            if task.next_run_time > now:
                delay = (task.next_run_time - now).total_seconds()
            else:
                delay = None

        msg = TaskMessage(
            queue=task.queue_name,
            args=task.task_args if task.task_args else [],
            kwargs=task.task_kwargs if task.task_kwargs else {},
            priority=task.priority,
            delay=delay,
            scheduled_task_id=task.id  
        )

        try:
            event_ids = await self.app.send_tasks([msg], asyncio=True)

            if event_ids:
                event_id = event_ids[0]

                if delay:
                    logger.info(
                        f"Scheduled task: scheduler_id={task.scheduler_id}, "
                        f"queue={task.queue_name}, "
                        f"event_id={event_id}, "
                        f"delay={delay:.1f}s"
                    )
                else:
                    logger.info(
                        f"Scheduled task (immediate): scheduler_id={task.scheduler_id}, "
                        f"queue={task.queue_name}, "
                        f"event_id={event_id}"
                    )
                return True
            else:
                logger.warning(f"Failed to send task {task.scheduler_id}: no event_id returned")
                return False

        except Exception as e:
            logger.error(f"Error sending task {task.scheduler_id}: {e}", exc_info=True)
            return False

    async def _update_tasks_after_scheduling(self, session: AsyncSession, tasks: List[ScheduledTask]):
        updates = []  
        once_task_ids = []  

        for task in tasks:
            if task.task_type == TaskType.ONCE.value:
                once_task_ids.append(task.id)
            else:
                last_run = task.next_run_time
                task.last_run_time = last_run
                next_run = task.calculate_next_run_time(from_time=last_run)
                next_trigger = task.calculate_trigger_time(next_run) if next_run else None

                updates.append((task.id, next_run, next_trigger, last_run))

        if updates:
            try:
                await ScheduledTask.batch_update_next_run_times(session, updates)
                logger.info(f"Updated {len(updates)} recurring tasks")
                for task_id, next_run, next_trigger, last_run in updates:
                    logger.info(
                        f"  Task {task_id}: next_run={next_run}, "
                        f"next_trigger={next_trigger}, last_run={last_run}"
                    )
            except Exception as e:
                logger.error(f"Failed to update recurring tasks: {e}", exc_info=True)

        if once_task_ids:
            try:
                await ScheduledTask.batch_disable_once_tasks(session, once_task_ids)
                logger.info(f"Disabled {len(once_task_ids)} once tasks")
            except Exception as e:
                logger.error(f"Failed to disable once tasks: {e}", exc_info=True)

    async def trigger_now(self, task_id: int) -> bool:
        session = self._session_factory()
        try:
            task = await ScheduledTask.get_by_id(session, task_id)
            if not task:
                logger.warning(f"Task {task_id} not found")
                return False

            msg = TaskMessage(
                queue=task.queue_name,
                args=task.task_args if task.task_args else [],
                kwargs=task.task_kwargs if task.task_kwargs else {},
                priority=task.priority,
                scheduled_task_id=task.id
            )

            try:
                event_ids = await self.app.send_tasks([msg], asyncio=True)

                if event_ids:
                    logger.info(
                        f"Manually triggered task {task.scheduler_id}: "
                        f"event_id={event_ids[0]}"
                    )
                    return True

            except Exception as e:
                logger.error(f"Error triggering task {task.scheduler_id}: {e}", exc_info=True)

            return False
        finally:
            await session.close()

    async def trigger_by_scheduler_id(self, scheduler_id: str) -> bool:
        session = self._session_factory()
        try:
            task = await ScheduledTask.get_by_scheduler_id(session, scheduler_id)
            if not task:
                logger.warning(f"Task with scheduler_id '{scheduler_id}' not found")
                return False

            return await self.trigger_now(task.id)
        finally:
            await session.close()


__all__ = ['TaskScheduler']



"""
Task Context - FastAPI风格的依赖注入
"""

from typing import TYPE_CHECKING, Optional, Any
from dataclasses import dataclass

if TYPE_CHECKING:
    from .app import Jettask


@dataclass
class TaskContext:
    """
    任务上下文信息

    通过类型注解自动注入到任务函数中：

    @app.task
    async def my_task(ctx: TaskContext, data: dict):
        print(f"Task ID: {ctx.event_id}")
        print(f"Task Name: {ctx.name}")
        if ctx.scheduled_task_id:
            print(f"Triggered by scheduled task: {ctx.scheduled_task_id}")
        # 访问元数据
        print(f"Priority: {ctx.metadata.get('priority')}")
        print(f"Delay: {ctx.metadata.get('delay')}")
        return data
    """
    event_id: str
    name: str
    trigger_time: float
    app: "Jettask"
    queue: Optional[str] = None
    worker_id: Optional[str] = None
    retry_count: int = 0
    scheduled_task_id: Optional[int] = None  
    group_name: Optional[str] = None  
    metadata: Optional[dict] = None  

    def __repr__(self) -> str:
        return f"TaskContext(event_id={self.event_id}, name={self.name}, queue={self.queue})"

    def acks(self, event_ids: list, offset: Optional[int] = None):
        if not event_ids:
            return

        if not self.queue:
            raise ValueError("TaskContext.queue is not set, cannot ACK")

        if not self.group_name:
            raise ValueError("TaskContext.group_name is not set, cannot ACK")

        ack_items = [
            (self.queue, event_id, self.group_name, offset)
            for event_id in event_ids
        ]

        self.app.ack(ack_items)


from ..utils.serializer import dumps_str, loads_str
import inspect
from dataclasses import dataclass
from typing import Any, Optional, TYPE_CHECKING, get_type_hints, Union

if TYPE_CHECKING:
    from .app import Jettask

from .context import TaskContext


@dataclass
class ExecuteResponse:
    delay: Optional[float] = None 
    urgent_retry: bool = False 
    reject: bool = False
    retry_time: Optional[float] = None


class Request:
    id: str = None
    name: str = None
    app: "Jettask" = None

    def __init__(self, *args, **kwargs) -> None:
        self._update(*args, **kwargs)

    def _update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)


class Task:
    _app: "Jettask" = None
    name: str = None
    queue: str = None
    trigger_time: float = None
    retry_config: Optional[dict] = None  

    def __call__(self, event_id: str, trigger_time: float, queue: str,
                 group_name: Optional[str] = None,
                 scheduled_task_id: Optional[int] = None,
                 metadata: Optional[dict] = None,
                 *args: Any, **kwds: Any) -> Any:
        injected_args, injected_kwargs = self._inject_dependencies(
            event_id, trigger_time, queue, group_name, scheduled_task_id, metadata, args, kwds
        )
        return self.run(*injected_args, **injected_kwargs)

    def _inject_dependencies(self, event_id: str, trigger_time: float, queue: str,
                            group_name: Optional[str], scheduled_task_id: Optional[int],
                            metadata: Optional[dict],
                            args: tuple, kwargs: dict) -> tuple:
        import logging
        logger = logging.getLogger(__name__)

        try:
            sig = inspect.signature(self.run)
            type_hints = get_type_hints(self.run)
            logger.debug(f"[TaskContext注入] 任务 {self.name} - 签名: {sig}, 类型提示: {type_hints}")
        except (ValueError, TypeError, NameError) as e:
            logger.warning(f"[TaskContext注入] 任务 {self.name} - 获取签名失败: {e}")
            return args, kwargs
        
        context = TaskContext(
            event_id=event_id,
            name=self.name,
            trigger_time=trigger_time,
            app=self._app,
            queue=queue,  
            scheduled_task_id=scheduled_task_id,  
            group_name=group_name,  
            metadata=metadata or {},  
        )
        
        params_list = list(sig.parameters.items())
        final_args = []
        final_kwargs = dict(kwargs)  
        args_list = list(args)
        args_consumed = 0  
        
        for idx, (param_name, param) in enumerate(params_list):
            if param_name == 'self':
                continue
            
            param_type = type_hints.get(param_name)
            
            if param.kind == param.KEYWORD_ONLY:
                if param_type is TaskContext and param_name not in final_kwargs:
                    final_kwargs[param_name] = context
                continue
            
            if param.kind in (param.POSITIONAL_ONLY, param.POSITIONAL_OR_KEYWORD):
                if param_type is TaskContext:
                    final_args.append(context)
                elif param_name in final_kwargs:
                    continue
                elif args_consumed < len(args_list):
                    final_args.append(args_list[args_consumed])
                    args_consumed += 1
                else:
                    break
        
        if args_consumed < len(args_list):
            final_args.extend(args_list[args_consumed:])

        logger.debug(f"[TaskContext注入] 任务 {self.name} - 注入后参数: args={final_args}, kwargs={list(final_kwargs.keys())}")

        return tuple(final_args), final_kwargs

    def run(self, *args, **kwargs):
        raise NotImplementedError("Tasks must define the run method.")

    @classmethod
    def bind_app(cls, app):
        cls._app = app

    @property
    def is_wildcard_queue(self) -> bool:
        from ..utils.queue_matcher import is_wildcard_pattern

        if not self.queue:
            return False

        return is_wildcard_pattern(self.queue)


    def on_before(self, event_id, pedding_count, args, kwargs) -> ExecuteResponse:
        return ExecuteResponse()

    def on_end(self, event_id, pedding_count, args, kwargs, result) -> ExecuteResponse:
        return ExecuteResponse()

    def on_success(self, event_id, args, kwargs, result) -> ExecuteResponse:
        return ExecuteResponse()

    def read_pending(
        self,
        queue: str = None,
        asyncio: bool = False,
    ):
        queue = queue or self.queue
        if asyncio:
            return self._get_pending(queue)
        return self._app.ep.read_pending(queue, queue)

    async def _get_pending(self, queue: str):
        return await self._app.ep.read_pending(queue, queue, asyncio=True)




"""
任务执行器

职责：
1. 从事件队列获取消息
2. 执行任务并管理生命周期
3. 限流控制和批处理
"""

import asyncio
import logging
import time
import os
from typing import Set
from collections import deque

from .core import ExecutorCore
from ..messaging.delay_queue import AsyncDelayQueue
from ..worker.lifecycle import WorkerManager
from ..utils.rate_limit.manager import RateLimiterManager

logger = logging.getLogger('app')


class TaskExecutor:
    """
    任务执行器 - 负责单个任务类型的执行

    这是真正执行任务的核心组件，无论是在单进程还是多进程模式下。

    职责：
    1. 从 event_queue 获取消息
    2. 应用限流控制
    3. 执行任务
    4. 批处理管道优化
    """

    def __init__(self, event_queue, app, task_name: str, worker_id: str, worker_state_manager, concurrency: int = 100):
        self.event_queue = event_queue
        self.app = app
        self.task_name = task_name
        self.concurrency = concurrency
        self.worker_id = worker_id
        self.worker_state_manager = worker_state_manager
        self.executor_core = ExecutorCore(
            app=app,
            task_name=task_name,
            concurrency=concurrency
        )

        if not hasattr(app, '_executor_core'):
            app._executor_core = self.executor_core

        task_obj = app.get_task_by_name(task_name)
        if task_obj and hasattr(task_obj, '_app') and not hasattr(task_obj._app, '_executor_core'):
            task_obj._app._executor_core = self.executor_core

        self._active_tasks: Set[asyncio.Task] = set()

        self.max_buffer_size = 5000

        self.delay_queue = AsyncDelayQueue()

        logger.debug(f"TaskExecutor initialized for task '{task_name}'")

    async def initialize(self):


        from jettask.utils.time_sync import init_time_sync
        time_sync = await init_time_sync(self.app.ep.async_redis_client)
        logger.debug(f"TimeSync initialized, offset={time_sync.get_offset():.6f}s")

        self.executor_core.rate_limiter_manager = RateLimiterManager(
            redis_client=self.app.ep.async_redis_client,
            worker_id=self.worker_id,
            redis_prefix=self.executor_core.prefix,
            worker_state_manager=self.worker_state_manager
        )
        logger.debug(f"RateLimiterManager initialized for worker {self.worker_id}")

        await self.executor_core.rate_limiter_manager.load_config_from_redis()

    async def run(self):
        logger.debug(f"TaskExecutor started for task '{self.task_name}'")

        tasks_batch = []

        try:
            while True:
                if hasattr(self.app, '_should_exit') and self.app._should_exit:
                    logger.debug(f"TaskExecutor[{self.task_name}] detected shutdown signal")
                    break

                if hasattr(os, 'getppid') and os.getppid() == 1:
                    logger.warning(f"TaskExecutor[{self.task_name}] parent died, exiting")
                    break

                current_time = time.time()

                expired_tasks = self.delay_queue.get_expired_tasks()
                for expired_task in expired_tasks:
                    event_data = expired_task.get('event_data', {})
                    event_data.pop("execute_at", None)
                    event_data.pop("is_delayed", None)

                    tasks_batch.append(expired_task)
                    logger.debug(
                        f"[{self.task_name}] Delayed task {expired_task.get('event_id', 'unknown')} "
                        f"expired and ready for execution"
                    )

                event = None
                try:
                    event = await asyncio.wait_for(self.event_queue.get(), timeout=0.1)
                except asyncio.TimeoutError:
                    pass
                
                if event:
                    event_data = event.get('event_data', {})
                    execute_at = event_data.get("execute_at")

                    if execute_at is not None:
                        current_time = time.time()

                        if execute_at > current_time:
                            delay = execute_at - current_time
                            self.delay_queue.put(event, delay)
                            logger.debug(
                                f"[{self.task_name}] Delayed task {event.get('event_id', 'unknown')} "
                                f"by {delay:.3f}s (execute_at={execute_at:.3f})"
                            )
                            continue
                        else:
                            event_data.pop("execute_at", None)
                            event_data.pop("is_delayed", None)
                            logger.debug(
                                f"[{self.task_name}] Delayed task {event.get('event_id', 'unknown')} "
                                f"already expired, executing immediately"
                            )

                    tasks_batch.append(event)
                    logger.debug(
                        f"[{self.task_name}] Got event: {event.get('event_id', 'unknown')}"
                    )

                if tasks_batch:
                    await self._process_batch(tasks_batch)
                    tasks_batch.clear()

                if self._should_flush_buffers(current_time):
                    asyncio.create_task(self.executor_core._flush_all_buffers())

                await self._smart_sleep()

        except asyncio.CancelledError:
            logger.debug(f"TaskExecutor[{self.task_name}] cancelled")
        except Exception as e:
            logger.error(f"TaskExecutor[{self.task_name}] error: {e}", exc_info=True)
        finally:
            await self.cleanup()

    async def _process_batch(self, tasks_batch):
        for event in tasks_batch:
            event_data = event.get('event_data', {})
            event_task_name = event_data.get("_task_name") or event_data.get("name")

            if not event_task_name:
                logger.error(f"No task_name in event {event.get('event_id')}")
                continue

            if event_task_name != self.task_name:
                logger.error(
                    f"Task name mismatch: {event_task_name} != {self.task_name}"
                )
                continue

            logger.debug(
                f"[{self.task_name}] Acquiring rate limit, event_id={event.get('event_id')}"
            )

            rate_limit_token = await self.executor_core.rate_limiter_manager.acquire(
                task_name=self.task_name,
                timeout=None
            )

            if not rate_limit_token:
                logger.error(f"Failed to acquire token for {self.task_name}")
                continue

            logger.debug(
                f"[{self.task_name}] Acquired token={rate_limit_token}, starting execution"
            )

            self.executor_core.batch_counter += 1

            async def execute_with_release(event_data, token):
                try:
                    await self.executor_core.execute_task(**event_data)
                finally:
                    await self.executor_core.rate_limiter_manager.release(
                        self.task_name, task_id=token
                    )

            task = asyncio.create_task(execute_with_release(event, rate_limit_token))
            self._active_tasks.add(task)
            task.add_done_callback(self._active_tasks.discard)

    def _should_flush_buffers(self, current_time: float) -> bool:
        buffer_full = (
            len(self.executor_core.pending_acks) >= self.max_buffer_size or
            len(self.executor_core.status_updates) >= self.max_buffer_size or
            len(self.executor_core.data_updates) >= self.max_buffer_size or
            len(self.executor_core.task_info_updates) >= self.max_buffer_size
        )

        if buffer_full:
            return True

        has_pending_data = (
            self.executor_core.pending_acks or
            self.executor_core.status_updates or
            self.executor_core.data_updates or
            self.executor_core.task_info_updates
        )

        if not has_pending_data:
            return False

        for data_type, config in self.executor_core.pipeline_config.items():
            time_since_flush = current_time - self.executor_core.last_pipeline_flush[data_type]

            if data_type == 'ack' and self.executor_core.pending_acks:
                if time_since_flush >= config['max_delay']:
                    return True
            elif data_type == 'task_info' and self.executor_core.task_info_updates:
                if time_since_flush >= config['max_delay']:
                    return True
            elif data_type == 'status' and self.executor_core.status_updates:
                if time_since_flush >= config['max_delay']:
                    return True
            elif data_type == 'data' and self.executor_core.data_updates:
                if time_since_flush >= config['max_delay']:
                    return True

        return False

    async def _smart_sleep(self):
        has_events = False

        if isinstance(self.event_queue, deque):
            has_events = bool(self.event_queue)
        elif isinstance(self.event_queue, asyncio.Queue):
            has_events = not self.event_queue.empty()

        if has_events:
            await asyncio.sleep(0)
        else:
            has_pending = (
                self.executor_core.pending_acks or
                self.executor_core.status_updates or
                self.executor_core.data_updates or
                self.executor_core.task_info_updates
            )

            if has_pending:
                await self.executor_core._flush_all_buffers()

            next_expire_time = self.delay_queue.get_next_expire_time()
            if next_expire_time is not None:
                current_time = time.time()
                sleep_time = max(0.001, min(0.1, next_expire_time - current_time))
                await asyncio.sleep(sleep_time)
            else:
                await asyncio.sleep(0.001)

    async def cleanup(self):
        logger.debug(f"TaskExecutor[{self.task_name}] cleaning up")

        self.delay_queue.clear()
        logger.debug("Delay queue cleared")

        if hasattr(self.app.ep, '_stop_reading'):
            self.app.ep._stop_reading = True

        if self._active_tasks:
            logger.debug(
                f"TaskExecutor[{self.task_name}] cancelling {len(self._active_tasks)} active tasks"
            )
            for task in self._active_tasks:
                if not task.done():
                    task.cancel()

            if self._active_tasks:
                try:
                    await asyncio.wait_for(
                        asyncio.gather(*self._active_tasks, return_exceptions=True),
                        timeout=0.2
                    )
                except asyncio.TimeoutError:
                    logger.warning(
                        f"TaskExecutor[{self.task_name}] some tasks did not complete in time"
                    )

        await self.executor_core.cleanup()

        logger.debug(f"TaskExecutor[{self.task_name}] stopped")


__all__ = ['TaskExecutor']



"""
队列通配符匹配工具

提供队列名称的通配符匹配功能，支持动态队列发现场景。
"""

import fnmatch
from typing import List, Dict, Set, Tuple


def is_wildcard_pattern(queue: str) -> bool:
    return '*' in queue or '?' in queue


def separate_wildcard_and_static_queues(queues: List[str]) -> Tuple[List[str], List[str]]:
    wildcard_patterns = []
    static_queues = []

    for queue in queues:
        if is_wildcard_pattern(queue):
            wildcard_patterns.append(queue)
        else:
            static_queues.append(queue)

    return wildcard_patterns, static_queues


def match_queue_to_pattern(queue: str, patterns: List[str]) -> str:
    for pattern in patterns:
        if fnmatch.fnmatch(queue, pattern):
            return pattern
    return None


def find_matching_tasks(
    queue: str,
    tasks_by_queue: Dict[str, List[str]],
    wildcard_mode: bool = False
) -> List[str]:
    task_names = tasks_by_queue.get(queue, [])

    if not task_names and wildcard_mode:
        for pattern, pattern_tasks in tasks_by_queue.items():
            if fnmatch.fnmatch(queue, pattern):
                task_names.extend(pattern_tasks)

    return task_names


def match_task_queue_to_patterns(
    task_queue: str,
    queue_patterns: List[str]
) -> bool:
    for queue_pattern in queue_patterns:
        if is_wildcard_pattern(queue_pattern):
            if fnmatch.fnmatch(task_queue, queue_pattern) or task_queue == queue_pattern:
                return True
        else:
            if task_queue == queue_pattern:
                return True

    return False


def discover_matching_queues(
    wildcard_patterns: List[str],
    all_queues: Set[str]
) -> Set[str]:
    matched_queues = set()

    for pattern in wildcard_patterns:
        for queue in all_queues:
            if fnmatch.fnmatch(queue, pattern):
                matched_queues.add(queue)

    return matched_queues



"""
任务注册器

统一管理所有任务定义
"""

from typing import Dict, Callable, Optional, Any, Set
from dataclasses import dataclass, field
import logging

logger = logging.getLogger('app')


@dataclass
class TaskDefinition:
    """任务定义"""
    name: str
    func: Callable
    queue: str = 'default'
    max_retries: int = 3
    timeout: int = 300
    retry_delay: int = 60
    priority: int = 0
    options: dict = field(default_factory=dict)

    def __post_init__(self):
        if not self.name:
            raise ValueError("Task name cannot be empty")
        if not callable(self.func):
            raise ValueError("Task func must be callable")
        if self.timeout <= 0:
            raise ValueError("Task timeout must be positive")


class TaskRegistry:
    """
    任务注册器

    职责：
    1. 任务注册和注销（本地内存）
    2. 任务查找
    3. 任务元数据管理
    4. Redis 任务注册（用于分布式发现）

    整合了：
    - TaskCenter的注册功能
    - RegistryManager的任务注册功能
    """

    def __init__(self, redis_client=None, async_redis_client=None, redis_prefix: str = 'jettask'):
        self.tasks: Dict[str, TaskDefinition] = {}
        self.redis = redis_client
        self.async_redis = async_redis_client
        self.redis_prefix = redis_prefix
        self.tasks_registry_key = f"{redis_prefix}:REGISTRY:TASKS"
        logger.debug("TaskRegistry initialized")

    def register(self,
                 name: str,
                 func: Callable,
                 queue: str = 'default',
                 **options) -> TaskDefinition:
        task_def = TaskDefinition(
            name=name,
            func=func,
            queue=queue,
            max_retries=options.get('max_retries', 3),
            timeout=options.get('timeout', 300),
            retry_delay=options.get('retry_delay', 60),
            priority=options.get('priority', 0),
            options=options
        )

        self.tasks[name] = task_def
        logger.info(f"Task registered: {name} -> queue: {queue}")

        return task_def

    def unregister(self, name: str) -> bool:
        if name in self.tasks:
            del self.tasks[name]
            logger.info(f"Task unregistered: {name}")
            return True

        logger.warning(f"Task not found for unregister: {name}")
        return False

    def get(self, name: str) -> Optional[TaskDefinition]:
        return self.tasks.get(name)

    def exists(self, name: str) -> bool:
        return name in self.tasks

    def list_all(self) -> Dict[str, TaskDefinition]:
        return self.tasks.copy()

    def list_by_queue(self, queue: str) -> Dict[str, TaskDefinition]:
        return {
            name: task_def
            for name, task_def in self.tasks.items()
            if task_def.queue == queue
        }

    def update_options(self, name: str, **options) -> bool:
        if name not in self.tasks:
            logger.warning(f"Task not found for update: {name}")
            return False

        task_def = self.tasks[name]

        if 'max_retries' in options:
            task_def.max_retries = options['max_retries']
        if 'timeout' in options:
            task_def.timeout = options['timeout']
        if 'retry_delay' in options:
            task_def.retry_delay = options['retry_delay']
        if 'priority' in options:
            task_def.priority = options['priority']
        if 'queue' in options:
            task_def.queue = options['queue']

        task_def.options.update(options)

        logger.info(f"Task options updated: {name}")
        return True

    def get_task_info(self, name: str) -> Optional[Dict[str, Any]]:
        task_def = self.get(name)
        if not task_def:
            return None

        return {
            'name': task_def.name,
            'queue': task_def.queue,
            'max_retries': task_def.max_retries,
            'timeout': task_def.timeout,
            'retry_delay': task_def.retry_delay,
            'priority': task_def.priority,
            'options': task_def.options
        }

    def count(self) -> int:
        return len(self.tasks)

    def clear(self):
        count = len(self.tasks)
        self.tasks.clear()
        logger.info(f"TaskRegistry cleared, {count} tasks removed")


    async def register_task_to_redis(self, task_name: str):
        if not self.async_redis:
            logger.warning("Redis client not configured, skipping Redis registration")
            return

        await self.async_redis.sadd(self.tasks_registry_key, task_name)
        logger.debug(f"Registered task to Redis: {task_name}")

    async def unregister_task_from_redis(self, task_name: str):
        if not self.async_redis:
            return

        await self.async_redis.srem(self.tasks_registry_key, task_name)
        logger.debug(f"Unregistered task from Redis: {task_name}")

    async def get_all_tasks_from_redis(self) -> Set[str]:
        if not self.async_redis:
            return set()

        return await self.async_redis.smembers(self.tasks_registry_key)

    async def get_task_count_from_redis(self) -> int:
        if not self.async_redis:
            return 0

        return await self.async_redis.scard(self.tasks_registry_key)



"""
告警相关的数据模型
"""
from typing import Optional, List, Dict, Any
from datetime import datetime
from pydantic import BaseModel, Field


class AlertRuleRequest(BaseModel):
    """告警规则请求模型"""
    rule_name: str = Field(..., description="规则名称")
    metric_type: str = Field(..., description="指标类型")
    threshold: float = Field(..., description="阈值")
    operator: str = Field(..., description="比较操作符", pattern="^(gt|gte|lt|lte|eq|ne)$")
    duration: int = Field(..., ge=1, description="持续时间（秒）")
    severity: str = Field(..., description="严重级别", pattern="^(low|medium|high|critical)$")
    enabled: bool = Field(default=True, description="是否启用")
    description: Optional[str] = Field(None, description="规则描述")


class AlertRuleCreate(BaseModel):
    """创建告警规则请求模型"""
    name: str = Field(..., description="规则名称")
    description: Optional[str] = Field(None, description="规则描述")
    metric_type: str = Field(..., description="监控指标类型")
    threshold: float = Field(..., description="告警阈值")
    comparison_operator: str = Field(..., description="比较操作符")
    evaluation_window: int = Field(..., description="评估窗口（秒）")
    severity: str = Field(..., description="告警级别")
    notification_channels: List[str] = Field(default=[], description="通知渠道")
    enabled: bool = Field(default=True, description="是否启用")


class AlertRuleUpdate(BaseModel):
    """更新告警规则请求模型"""
    name: Optional[str] = Field(None, description="规则名称")
    description: Optional[str] = Field(None, description="规则描述")
    metric_type: Optional[str] = Field(None, description="监控指标类型")
    threshold: Optional[float] = Field(None, description="告警阈值")
    comparison_operator: Optional[str] = Field(None, description="比较操作符")
    evaluation_window: Optional[int] = Field(None, description="评估窗口（秒）")
    severity: Optional[str] = Field(None, description="告警级别")
    notification_channels: Optional[List[str]] = Field(None, description="通知渠道")
    enabled: Optional[bool] = Field(None, description="是否启用")


class AlertRule(BaseModel):
    """告警规则模型"""
    id: str = Field(..., description="规则ID")
    name: str = Field(..., description="规则名称")
    description: Optional[str] = Field(None, description="规则描述")
    metric_type: str = Field(..., description="监控指标类型")
    threshold: float = Field(..., description="告警阈值")
    comparison_operator: str = Field(..., description="比较操作符")
    evaluation_window: int = Field(..., description="评估窗口（秒）")
    severity: str = Field(..., description="告警级别")
    notification_channels: List[str] = Field(default=[], description="通知渠道")
    enabled: bool = Field(default=True, description="是否启用")
    created_at: datetime = Field(..., description="创建时间")
    updated_at: Optional[datetime] = Field(None, description="更新时间")
    
    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat() if v else None
        }


class AlertInstance(BaseModel):
    """告警实例模型"""
    id: str = Field(..., description="告警实例ID")
    rule_id: str = Field(..., description="规则ID")
    rule_name: str = Field(..., description="规则名称")
    status: str = Field(..., description="告警状态", pattern="^(active|resolved|suppressed)$")
    severity: str = Field(..., description="告警级别")
    message: str = Field(..., description="告警消息")
    metric_value: float = Field(..., description="当前指标值")
    threshold: float = Field(..., description="阈值")
    started_at: datetime = Field(..., description="开始时间")
    resolved_at: Optional[datetime] = Field(None, description="解决时间")
    acknowledged_at: Optional[datetime] = Field(None, description="确认时间")
    acknowledged_by: Optional[str] = Field(None, description="确认人")
    namespace: str = Field(..., description="命名空间")
    labels: Dict[str, str] = Field(default={}, description="标签")
    
    class Config:
        json_encoders = {
            datetime: lambda v: v.isoformat() if v else None
        }


class AlertSummary(BaseModel):
    """告警摘要模型"""
    total_alerts: int = Field(default=0, description="总告警数")
    active_alerts: int = Field(default=0, description="活跃告警数")
    resolved_alerts: int = Field(default=0, description="已解决告警数")
    critical_alerts: int = Field(default=0, description="严重告警数")
    high_alerts: int = Field(default=0, description="高级告警数")
    medium_alerts: int = Field(default=0, description="中级告警数")
    low_alerts: int = Field(default=0, description="低级告警数")


"""
数据库模型定义

所有表的 SQLAlchemy 模型定义
"""

from .task import Task
from .task_run import TaskRun
from .scheduled_task import ScheduledTask
from .task_metrics_minute import TaskMetricsMinute
from .task_runs_metrics_minute import TaskRunsMetricsMinute
from .sql_history import SQLHistory

from jettask.core.enums import TaskType, TaskStatus

__all__ = [
    'Task',
    'TaskRun',
    'ScheduledTask',
    'TaskMetricsMinute',
    'TaskRunsMetricsMinute',
    'SQLHistory',
    'TaskType',
    'TaskStatus',
]



"""
概览服务层
处理系统概览和健康检查相关的业务逻辑
"""
from datetime import datetime, timedelta, timezone
from typing import Dict, Any, List, Optional
import logging
import time
import traceback
from sqlalchemy import text
from sqlalchemy.ext.asyncio import AsyncSession

from jettask.schemas import TimeRangeQuery
from jettask.db.connector import get_async_redis_client, get_pg_engine_and_factory
from jettask.webui.services.base_service import BaseService, TimeRangeResult

logger = logging.getLogger(__name__)



class OverviewService(BaseService):
    """概览服务类"""
    
    @staticmethod
    def get_root_info() -> Dict[str, Any]:
        return {
            "success": True,
            "data": {
                "service": "JetTask WebUI API",
                "version": "v1",
                "status": "running",
                "timestamp": datetime.now(timezone.utc).isoformat()
            }
        }
    
    @staticmethod
    def get_health_status() -> Dict[str, Any]:
        return {
            "status": "healthy",
            "timestamp": datetime.now(timezone.utc).isoformat()
        }
    
    @staticmethod
    async def get_system_stats(namespace: str) -> Dict[str, Any]:
        import os
        redis_url = os.environ.get('JETTASK_REDIS_URL', 'redis://localhost:6379/0')
        redis_client = get_async_redis_client(redis_url, decode_responses=True)
        
        try:
            stats = {
                'namespace': namespace,
                'queues': 0,
                'tasks': 0,
                'delayed_tasks': 0,
                'workers': 0
            }
            
            from jettask.messaging.registry import QueueRegistry
            from jettask.task.task_registry import TaskRegistry
            from jettask.worker.lifecycle import WorkerManager as WorkerRegistry

            sync_redis_client = get_sync_redis_client(redis_url, decode_responses=True)

            queue_registry = QueueRegistry(
                redis_client=sync_redis_client,
                async_redis_client=redis_client,
                redis_prefix=namespace
            )

            task_registry = TaskRegistry(
                redis_client=None,
                async_redis_client=redis_client,
                redis_prefix=namespace
            )

            worker_registry = WorkerRegistry(
                redis_client=None,
                async_redis_client=redis_client,
                redis_prefix=namespace
            )

            stats['queues'] = await queue_registry.get_queue_count()

            stats['tasks'] = await task_registry.get_task_count_from_redis()

            stats['delayed_tasks'] = 0

            stats['workers'] = await worker_registry.get_worker_count()
            
            return stats
            
        finally:
            await redis_client.aclose()
    
    @staticmethod
    async def get_dashboard_stats(
        namespace: str,
        time_range: str = "24h",
        queues: Optional[str] = None
    ) -> Dict[str, Any]:
        import os

        pg_url = os.environ.get('JETTASK_PG_URL')
        if not pg_url:
            return {
                "success": True,
                "data": _get_empty_dashboard_stats()
            }

        end_time = datetime.now(timezone.utc)
        start_time = BaseService._parse_time_range(time_range, end_time)

        queue_filter, queue_list, queue_params = _build_queue_filter_and_params(queues)

        _, session_factory = get_pg_engine_and_factory(pg_url)
        async with session_factory() as session:
            stats_data = await _get_task_statistics(
                session, namespace, start_time, end_time,
                queue_filter, queue_params
            )
            
            throughput = await _calculate_throughput(
                session, namespace, queue_filter, queue_params
            )
            
            distribution_params = {
                'namespace': namespace,
                'start_time': start_time,
                'end_time': end_time,
                **queue_params
            }
            distribution_data = await _get_task_distribution(
                session, namespace, start_time, end_time,
                queue_filter, distribution_params
            )
            
            return {
                "success": True,
                "data": {
                    **stats_data,
                    "throughput": throughput,
                    "distribution": distribution_data
                }
            }
    
    @staticmethod
    async def get_top_queues(
        namespace: str,
        metric: str = "backlog",
        limit: int = 10,
        time_range: str = "24h",
        queues: Optional[str] = None
    ) -> Dict[str, Any]:
        if metric == "backlog":
            return await _get_top_backlog_queues(namespace, limit, time_range, queues)
        elif metric == "error":
            return await _get_top_error_queues(namespace, limit, time_range, queues)
        else:
            raise ValueError(f"不支持的指标类型: {metric}")
    
    @staticmethod
    async def get_dashboard_overview_stats(
        namespace: str,
        query: TimeRangeQuery
    ) -> Dict[str, Any]:
        import os

        pg_url = os.environ.get('JETTASK_PG_URL')
        if not pg_url:
            return _get_empty_overview_stats()

        time_range_result = BaseService._parse_time_range_query(query)

        queue_list = query.queues if hasattr(query, 'queues') and query.queues else None
        queue_filter, _, queue_params = _build_queue_filter_and_params(queue_list)

        _, session_factory = get_pg_engine_and_factory(pg_url)
        async with session_factory() as session:
            result = await _execute_overview_query(
                session, namespace, time_range_result,
                queue_filter, queue_params
            )
            
            return _format_overview_data(result, time_range_result.granularity)



def _build_queue_filter_and_params(queues: Optional[List[str]] = None):
    queue_list = []
    if queues:
        if isinstance(queues, str):
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
        elif isinstance(queues, list):
            queue_list = [q.strip() for q in queues if q and q.strip()]
        else:
            queue_list = []
    
    queue_filter = ""
    queue_params = {}
    
    if queue_list:
        queue_placeholders = ','.join([f':queue_{i}' for i in range(len(queue_list))])
        queue_filter = f"AND t.queue IN ({queue_placeholders})"
        
        for i, queue in enumerate(queue_list):
            queue_params[f'queue_{i}'] = queue
    
    return queue_filter, queue_list, queue_params


def _get_empty_dashboard_stats():
    return {
        "total_tasks": 0,
        "completed_tasks": 0,
        "failed_tasks": 0,
        "running_tasks": 0,
        "pending_tasks": 0,
        "success_rate": 0,
        "throughput": 0,
        "avg_processing_time": 0,
        "total_queues": 0,
        "distribution": [{'type': '暂无数据', 'value': 1, 'queue': '', 'status': 'empty'}]
    }


def _get_empty_overview_stats():
    return {
        "task_trend": [],
        "concurrency": [],
        "processing_time": [],
        "creation_latency": [],
        "granularity": "minute"
    }


async def _get_task_statistics(session, namespace, start_time, end_time, queue_filter, queue_params):
    stats_sql = text(f"""
        WITH task_stats AS (
            SELECT 
                t.stream_id,
                t.created_at,
                t.queue,
                tr.status,
                tr.execution_time,
                tr.end_time
            FROM tasks t
            LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
            WHERE t.namespace = :namespace
                AND t.created_at >= :start_time
                AND t.created_at <= :end_time
                {queue_filter}
        )
        SELECT 
            COUNT(DISTINCT stream_id) as total_tasks,
            COUNT(DISTINCT CASE WHEN status = 'success' THEN stream_id END) as completed_tasks,
            COUNT(DISTINCT CASE WHEN status = 'error' THEN stream_id END) as failed_tasks,
            COUNT(DISTINCT CASE WHEN status = 'running' THEN stream_id END) as running_tasks,
            COUNT(DISTINCT CASE WHEN status IS NULL OR status = 'pending' THEN stream_id END) as pending_tasks,
            COUNT(DISTINCT queue) as total_queues,
            AVG(CASE WHEN status = 'success' AND execution_time IS NOT NULL 
                THEN execution_time END) as avg_execution_time
        FROM task_stats
    """)
    
    all_params = {
        'namespace': namespace,
        'start_time': start_time,
        'end_time': end_time,
        **queue_params
    }
    
    result = await session.execute(stats_sql, all_params)
    row = result.first()
    
    if row:
        avg_execution_time = row.avg_execution_time or 0
        success_rate = round((row.completed_tasks / row.total_tasks * 100) if row.total_tasks > 0 else 0, 1)
        
        return {
            "total_tasks": row.total_tasks or 0,
            "completed_tasks": row.completed_tasks or 0,
            "failed_tasks": row.failed_tasks or 0,
            "running_tasks": row.running_tasks or 0,
            "pending_tasks": row.pending_tasks or 0,
            "success_rate": success_rate,
            "avg_processing_time": round(avg_execution_time * 1000 if avg_execution_time else 0, 1),
            "total_queues": row.total_queues or 0
        }
    
    return _get_empty_dashboard_stats()


async def _calculate_throughput(session, namespace, queue_filter, queue_params):
    recent_end_time = datetime.now(timezone.utc)
    throughput = 0
    
    time_windows = [
        (5, "最近5分钟"),
        (10, "最近10分钟"), 
        (30, "最近30分钟"),
        (60, "最近1小时")
    ]
    
    for window_minutes, window_desc in time_windows:
        recent_start_time = recent_end_time - timedelta(minutes=window_minutes)
        
        recent_query = text(f"""
            SELECT COUNT(DISTINCT t.stream_id) as recent_completed
            FROM tasks t
            LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
            WHERE t.namespace = :namespace
                AND tr.status = 'success'
                AND tr.end_time >= :recent_start_time
                AND tr.end_time <= :recent_end_time
                {queue_filter}
        """)
        
        throughput_params = {
            'namespace': namespace,
            'recent_start_time': recent_start_time,
            'recent_end_time': recent_end_time,
            **queue_params
        }
        
        recent_result = await session.execute(recent_query, throughput_params)
        recent_row = recent_result.first()
        recent_completed = recent_row.recent_completed if recent_row else 0
        
        if recent_completed >= 5:
            throughput = round(recent_completed / window_minutes, 1)
            logger.info(f"使用{window_desc}计算吞吐量: {recent_completed}个任务/{window_minutes}分钟 = {throughput}任务/分钟")
            break
        elif recent_completed > 0:
            throughput = round(recent_completed / window_minutes, 1)
    
    return throughput


async def _get_task_distribution(session, namespace, start_time, end_time, queue_filter, query_params):
    distribution_sql = text(f"""
        SELECT 
            t.queue,
            COUNT(DISTINCT t.stream_id) as count
        FROM tasks t
        WHERE t.namespace = :namespace
            AND t.created_at >= :start_time
            AND t.created_at <= :end_time
            {queue_filter}
        GROUP BY t.queue
        ORDER BY count DESC, t.queue
    """)
    
    distribution_result = await session.execute(distribution_sql, query_params)
    
    distribution_data = []
    for row in distribution_result.fetchall():
        if row.count > 0:
            distribution_data.append({
                'type': row.queue,
                'value': row.count,
                'queue': row.queue,
                'status': 'all'
            })
    
    if not distribution_data:
        distribution_data = [
            {'type': '暂无数据', 'value': 1, 'queue': '', 'status': 'empty'}
        ]
    
    return distribution_data


async def _get_top_backlog_queues(namespace, limit, time_range, queues):
    import os

    pg_url = os.environ.get('JETTASK_PG_URL')
    if not pg_url:
        return {"success": True, "data": []}

    end_time = datetime.now(timezone.utc)
    start_time = BaseService._parse_time_range(time_range, end_time)

    _, session_factory = get_pg_engine_and_factory(pg_url)
    async with session_factory() as session:
        return await _get_top_backlog_from_tasks(session, namespace, limit)


async def _get_top_backlog_from_tasks(session, namespace, limit):
    task_sql = text("""
        SELECT 
            t.queue,
            COUNT(DISTINCT t.stream_id) as backlog,
            CASE 
                WHEN COUNT(DISTINCT t.stream_id) > 1000 THEN 'critical'
                WHEN COUNT(DISTINCT t.stream_id) > 500 THEN 'warning'
                ELSE 'normal'
            END as status
        FROM tasks t
        LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
        WHERE t.namespace = :namespace
            AND (tr.stream_id IS NULL OR tr.status = 'pending')
            AND t.created_at > NOW() - INTERVAL '24 hour'
        GROUP BY t.queue
        ORDER BY backlog DESC
        LIMIT :limit
    """)
    
    result = await session.execute(task_sql, {
        'namespace': namespace,
        'limit': limit
    })
    
    backlog_queues = []
    for row in result.fetchall():
        backlog_queues.append({
            "queue": row.queue,
            "backlog": int(row.backlog),
            "status": row.status
        })
    
    return {"success": True, "data": backlog_queues}


async def _get_top_error_queues(namespace, limit, time_range, queues):
    import os

    pg_url = os.environ.get('JETTASK_PG_URL')
    if not pg_url:
        return {"success": True, "data": []}

    end_time = datetime.now(timezone.utc)
    start_time = BaseService._parse_time_range(time_range, end_time)

    _, session_factory = get_pg_engine_and_factory(pg_url)
    async with session_factory() as session:
        queue_list = []
        if queues:
            queue_list = [q.strip() for q in queues.split(',') if q.strip()]
        
        if queue_list:
            error_sql = text("""
                WITH queue_stats AS (
                    SELECT 
                        t.queue,
                        COUNT(DISTINCT t.stream_id) as total,
                        COUNT(DISTINCT CASE WHEN tr.status = 'error' THEN t.stream_id END) as errors
                    FROM tasks t
                    LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
                    WHERE t.namespace = :namespace
                        AND t.created_at >= :start_time
                        AND t.created_at <= :end_time
                        AND t.queue = ANY(:queues)
                    GROUP BY t.queue
                )
                SELECT 
                    queue,
                    errors,
                    total,
                    CASE 
                        WHEN total > 0 THEN ROUND(errors::numeric / total * 100, 1)
                        ELSE 0 
                    END as error_rate
                FROM queue_stats
                WHERE errors > 0
                ORDER BY error_rate DESC, errors DESC
                LIMIT :limit
            """)
            
            result = await session.execute(error_sql, {
                'namespace': namespace,
                'start_time': start_time,
                'end_time': end_time,
                'queues': queue_list,
                'limit': limit
            })
        else:
            error_sql = text("""
                WITH queue_stats AS (
                    SELECT 
                        t.queue,
                        COUNT(DISTINCT t.stream_id) as total,
                        COUNT(DISTINCT CASE WHEN tr.status = 'error' THEN t.stream_id END) as errors
                    FROM tasks t
                    LEFT JOIN task_runs tr ON t.stream_id = tr.stream_id
                    WHERE t.namespace = :namespace
                        AND t.created_at >= :start_time
                        AND t.created_at <= :end_time
                    GROUP BY t.queue
                )
                SELECT 
                    queue,
                    errors,
                    total,
                    CASE 
                        WHEN total > 0 THEN ROUND(errors::numeric / total * 100, 1)
                        ELSE 0 
                    END as error_rate
                FROM queue_stats
                WHERE errors > 0
                ORDER BY error_rate DESC, errors DESC
                LIMIT :limit
            """)
            
            result = await session.execute(error_sql, {
                'namespace': namespace,
                'start_time': start_time,
                'end_time': end_time,
                'limit': limit
            })
        
        error_queues = []
        for row in result.fetchall():
            error_queues.append({
                "queue": row.queue,
                "errors": int(row.errors),
                "total": int(row.total),
                "error_rate": float(row.error_rate)
            })
        
        return {"success": True, "data": error_queues}


async def _execute_overview_query(session, namespace, time_range_result, queue_filter, queue_params):
    interval_seconds = time_range_result.interval_seconds
    
    sql = text(f"""
        WITH time_series AS (
            SELECT to_timestamp(FLOOR(EXTRACT(epoch FROM ts) / {interval_seconds}) * {interval_seconds}) AS time_bucket
            FROM generate_series(
                :start_time ::timestamptz,
                :end_time ::timestamptz + INTERVAL '{interval_seconds} seconds',
                INTERVAL '{interval_seconds} seconds'
            ) AS ts
        ),
        enqueue_counts AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM t.created_at) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                COUNT(DISTINCT t.stream_id) as enqueued
            FROM tasks t
            WHERE t.namespace = :namespace
                AND t.created_at >= :start_time
                AND t.created_at <= :end_time
                {queue_filter}
            GROUP BY time_bucket
        ),
        complete_counts AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM tr.end_time) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                COUNT(DISTINCT t.stream_id) as completed
            FROM task_runs tr
            JOIN tasks t ON tr.stream_id = t.stream_id
            WHERE t.namespace = :namespace
                AND tr.end_time >= :start_time
                AND tr.end_time <= :end_time
                AND tr.status = 'success'
                AND t.created_at >= :start_time
                AND t.created_at <= :end_time
                {queue_filter}
            GROUP BY time_bucket
        ),
        failed_counts AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM tr.end_time) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                COUNT(DISTINCT t.stream_id) as failed
            FROM task_runs tr
            JOIN tasks t ON tr.stream_id = t.stream_id
            WHERE t.namespace = :namespace
                AND tr.end_time >= :start_time
                AND tr.end_time <= :end_time
                AND tr.status = 'error'
                AND t.created_at >= :start_time
                AND t.created_at <= :end_time
                {queue_filter}
            GROUP BY time_bucket
        ),
        concurrency_data AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM tr.start_time) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                COUNT(DISTINCT t.stream_id) as concurrent_tasks
            FROM task_runs tr
            JOIN tasks t ON tr.stream_id = t.stream_id
            WHERE t.namespace = :namespace
                AND tr.start_time >= :start_time
                AND tr.start_time <= :end_time
                AND tr.start_time IS NOT NULL
                AND tr.end_time IS NOT NULL
                AND t.created_at >= :start_time
                AND t.created_at <= :end_time
                {queue_filter}
            GROUP BY time_bucket
        ),
        processing_time_data AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM tr.end_time) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                AVG(CASE WHEN tr.status = 'success' AND tr.execution_time > 0 
                    THEN tr.execution_time END) as avg_processing_time,
                PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY 
                    CASE WHEN tr.status = 'success' AND tr.execution_time > 0 
                    THEN tr.execution_time END) as p50_processing_time,
                PERCENTILE_CONT(0.9) WITHIN GROUP (ORDER BY 
                    CASE WHEN tr.status = 'success' AND tr.execution_time > 0 
                    THEN tr.execution_time END) as p90_processing_time
            FROM task_runs tr
            JOIN tasks t ON tr.stream_id = t.stream_id
            WHERE t.namespace = :namespace
                AND tr.end_time >= :start_time
                AND tr.end_time <= :end_time
                AND tr.status = 'success'
                {queue_filter}
            GROUP BY time_bucket
        ),
        creation_latency_data AS (
            SELECT 
                to_timestamp(FLOOR(EXTRACT(epoch FROM tr.start_time) / {interval_seconds}) * {interval_seconds}) AS time_bucket,
                AVG(EXTRACT(EPOCH FROM (tr.start_time - t.created_at))) as avg_creation_latency,
                PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY 
                    EXTRACT(EPOCH FROM (tr.start_time - t.created_at))) as p50_creation_latency,
                PERCENTILE_CONT(0.9) WITHIN GROUP (ORDER BY 
                    EXTRACT(EPOCH FROM (tr.start_time - t.created_at))) as p90_creation_latency
            FROM task_runs tr
            JOIN tasks t ON tr.stream_id = t.stream_id
            WHERE t.namespace = :namespace
                AND tr.start_time >= :start_time
                AND tr.start_time <= :end_time
                AND tr.start_time IS NOT NULL
                {queue_filter}
            GROUP BY time_bucket
        )
        SELECT 
            ts.time_bucket,
            COALESCE(eq.enqueued, 0) as enqueued,
            COALESCE(cc.completed, 0) as completed,
            COALESCE(fc.failed, 0) as failed,
            COALESCE(cd.concurrent_tasks, 0) as concurrent_tasks,
            ROUND(ptd.avg_processing_time::numeric, 6) as avg_processing_time,
            ROUND(ptd.p50_processing_time::numeric, 6) as p50_processing_time,
            ROUND(ptd.p90_processing_time::numeric, 6) as p90_processing_time,
            ROUND(cld.avg_creation_latency::numeric, 3) as avg_creation_latency,
            ROUND(cld.p50_creation_latency::numeric, 3) as p50_creation_latency,
            ROUND(cld.p90_creation_latency::numeric, 3) as p90_creation_latency
        FROM time_series ts
        LEFT JOIN enqueue_counts eq ON ts.time_bucket = eq.time_bucket
        LEFT JOIN complete_counts cc ON ts.time_bucket = cc.time_bucket
        LEFT JOIN failed_counts fc ON ts.time_bucket = fc.time_bucket
        LEFT JOIN concurrency_data cd ON ts.time_bucket = cd.time_bucket
        LEFT JOIN processing_time_data ptd ON ts.time_bucket = ptd.time_bucket
        LEFT JOIN creation_latency_data cld ON ts.time_bucket = cld.time_bucket
        ORDER BY ts.time_bucket
    """)
    
    query_params = {
        'namespace': namespace,
        'start_time': time_range_result.start_time,
        'end_time': time_range_result.end_time,
        **queue_params
    }
    
    result = await session.execute(sql, query_params)
    return result.fetchall()


def _format_overview_data(rows, granularity):
    task_trend = []
    concurrency = []
    processing_time = []
    creation_latency = []
    
    end_index = len(rows) - 1
    
    for idx, row in enumerate(rows):
        time_str = row.time_bucket.isoformat()
        
        enqueued_val = row.enqueued if row.enqueued > 0 or idx == 0 or idx == end_index else None
        completed_val = row.completed if row.completed > 0 or idx == 0 or idx == end_index else None
        failed_val = row.failed if row.failed > 0 or idx == 0 or idx == end_index else None
        
        task_trend.extend([
            {'time': time_str, 'value': enqueued_val, 'metric': '入队速率'},
            {'time': time_str, 'value': completed_val, 'metric': '完成速率'},
            {'time': time_str, 'value': failed_val, 'metric': '失败数'}
        ])
        
        concurrency.append({
            'time': time_str,
            'value': row.concurrent_tasks or 0,
            'metric': '并发任务数'
        })
        
        _add_processing_time_data(processing_time, time_str, row, idx, end_index)
        
        _add_creation_latency_data(creation_latency, time_str, row, idx, end_index)
    
    return {
        "task_trend": task_trend,
        "concurrency": concurrency,
        "processing_time": processing_time,
        "creation_latency": creation_latency,
        "granularity": granularity
    }


def _add_processing_time_data(processing_time, time_str, row, idx, end_index):
    if row.avg_processing_time is not None:
        avg_time_val = round(float(row.avg_processing_time * 1000), 1)
    else:
        avg_time_val = None if idx != 0 and idx != end_index else 0
        
    if row.p50_processing_time is not None:
        p50_time_val = round(float(row.p50_processing_time * 1000), 1)
    else:
        p50_time_val = None if idx != 0 and idx != end_index else 0
        
    if row.p90_processing_time is not None:
        p90_time_val = round(float(row.p90_processing_time * 1000), 1)
    else:
        p90_time_val = None if idx != 0 and idx != end_index else 0
    
    processing_time.extend([
        {'time': time_str, 'value': avg_time_val, 'metric': '平均处理时间'},
        {'time': time_str, 'value': p50_time_val, 'metric': 'P50处理时间'},
        {'time': time_str, 'value': p90_time_val, 'metric': 'P90处理时间'}
    ])


def _add_creation_latency_data(creation_latency, time_str, row, idx, end_index):
    if row.avg_creation_latency is not None:
        avg_latency_val = round(float(row.avg_creation_latency), 3)
    else:
        avg_latency_val = None if idx != 0 and idx != end_index else 0
        
    if row.p50_creation_latency is not None:
        p50_latency_val = round(float(row.p50_creation_latency), 3)
    else:
        p50_latency_val = None if idx != 0 and idx != end_index else 0
        
    if row.p90_creation_latency is not None:
        p90_latency_val = round(float(row.p90_creation_latency), 3)
    else:
        p90_latency_val = None if idx != 0 and idx != end_index else 0
    
    creation_latency.extend([
        {'time': time_str, 'value': avg_latency_val, 'metric': '平均执行延时'},
        {'time': time_str, 'value': p50_latency_val, 'metric': 'P50执行延时'},
        {'time': time_str, 'value': p90_latency_val, 'metric': 'P90执行延时'}
    ])


"""
App importer module inspired by Uvicorn's import logic.
支持多种方式导入 Jettask 应用实例。
"""

import os
import sys
import importlib
import importlib.util
from pathlib import Path
from typing import Optional, Any
import inspect


class AppImporter:
    """应用导入器，支持多种导入方式"""
    
    DEFAULT_APP_NAMES = ['app', 'application', 'jettask_app']
    
    DEFAULT_FILES = ['app.py', 'main.py', 'server.py', 'worker.py']
    
    @classmethod
    def import_from_string(cls, import_str: str) -> Any:
        if ':' in import_str:
            module_str, app_name = import_str.rsplit(':', 1)
        else:
            module_str = import_str
            app_name = None
        
        if '/' in module_str or module_str.endswith('.py') or os.path.isdir(module_str):
            module = cls._import_from_file(module_str)
        else:
            module = cls._import_from_module(module_str)
        
        if app_name:
            if '.' in app_name or '(' in app_name:
                app = cls._evaluate_app_expression(module, app_name)
            else:
                app = getattr(module, app_name)
        else:
            app = cls._find_app_in_module(module)
            if not app:
                raise ImportError(
                    f"Cannot find Jettask app in {module_str}. "
                    f"Tried names: {', '.join(cls.DEFAULT_APP_NAMES)}"
                )
        
        return app
    
    @classmethod
    def _import_from_file(cls, file_path: str):
        path = Path(file_path)
        
        if not path.is_absolute():
            path = Path.cwd() / path
        
        if path.is_dir():
            init_file = path / '__init__.py'
            if not init_file.exists():
                raise ImportError(
                    f"Directory {path} does not contain __init__.py. "
                    f"Cannot import as a Python package."
                )
            py_file = init_file
            module_name = path.name
            parent_dir = path.parent
        else:
            if path.suffix == '.py':
                path = path.with_suffix('')
            
            py_file = path.with_suffix('.py')
            if not py_file.exists():
                raise ImportError(f"File not found: {py_file}")
            
            module_name = path.name
            parent_dir = path.parent
        
        sys.path.insert(0, str(parent_dir))
        
        try:
            spec = importlib.util.spec_from_file_location(
                module_name, 
                str(py_file)
            )
            if spec and spec.loader:
                module = importlib.util.module_from_spec(spec)
                sys.modules[module_name] = module
                try:
                    spec.loader.exec_module(module)
                    return module
                except Exception as e:
                    import traceback
                    tb_lines = traceback.format_exc().splitlines()
                    
                    user_error_lines = [line for line in tb_lines 
                                       if str(py_file) in line or module_name in line]
                    
                    raise ImportError(
                        f"Failed to load module from {py_file}.\n"
                        f"Error in user code: {str(e)}\n"
                        f"Check the file for syntax errors or missing dependencies."
                    ) from e
            else:
                raise ImportError(f"Cannot create module spec for {py_file}")
        finally:
            if str(parent_dir) in sys.path:
                sys.path.remove(str(parent_dir))
    
    @classmethod
    def _import_from_module(cls, module_str: str):
        try:
            return importlib.import_module(module_str)
        except ImportError as e:
            original_error = str(e)
            
            if '.' not in sys.path:
                sys.path.insert(0, '.')
                try:
                    return importlib.import_module(module_str)
                except ImportError as retry_error:
                    import traceback
                    tb_str = traceback.format_exc()
                    
                    if f"No module named '{module_str}'" in str(retry_error):
                        raise ImportError(
                            f"Module '{module_str}' not found. "
                            f"Make sure the module exists and is in the Python path."
                        ) from retry_error
                    else:
                        raise ImportError(
                            f"Failed to import module '{module_str}'. "
                            f"The module was found but contains errors:\n{original_error}"
                        ) from retry_error
            raise
    
    @classmethod
    def _find_app_in_module(cls, module) -> Optional[Any]:
        from ..core.app import Jettask
        
        for name in cls.DEFAULT_APP_NAMES:
            if hasattr(module, name):
                obj = getattr(module, name)
                if isinstance(obj, Jettask):
                    return obj
        
        for name in dir(module):
            if not name.startswith('_'):
                obj = getattr(module, name, None)
                if isinstance(obj, Jettask):
                    return obj
        
        return None
    
    @classmethod
    def _evaluate_app_expression(cls, module, expression: str):
        namespace = {'__builtins__': {}}
        namespace.update(vars(module))
        
        try:
            return eval(expression, namespace)
        except Exception as e:
            raise ImportError(f"Cannot evaluate expression '{expression}': {e}")
    
    @classmethod
    def auto_discover(cls) -> Optional[Any]:
        from ..core.app import Jettask
        
        env_app = os.getenv('JETTASK_APP')
        if env_app:
            try:
                return cls.import_from_string(env_app)
            except Exception:
                pass
        
        for filename in cls.DEFAULT_FILES:
            file_path = Path.cwd() / filename
            if file_path.exists():
                try:
                    module = cls._import_from_file(str(file_path))
                    app = cls._find_app_in_module(module)
                    if app:
                        return app
                except Exception:
                    continue
        
        for py_file in Path.cwd().glob('*.py'):
            if py_file.name.startswith('_'):
                continue
            try:
                module = cls._import_from_file(str(py_file))
                app = cls._find_app_in_module(module)
                if app:
                    return app
            except Exception:
                continue
        
        return None
    
    @classmethod
    def get_app_info(cls, app) -> dict:
        from ..core.app import Jettask
        
        if not isinstance(app, Jettask):
            return {'error': 'Not a Jettask instance'}
        
        info = {
            'type': 'Jettask',
            'redis_url': getattr(app, 'redis_url', 'Not configured'),
            'redis_prefix': getattr(app, 'redis_prefix', 'jettask'),
            'tasks': len(getattr(app, '_tasks', {})),
        }
        
        if hasattr(app, '_tasks'):
            info['task_names'] = list(app._tasks.keys())
        
        return info


def import_app(import_str: Optional[str] = None) -> Any:
    if import_str:
        return AppImporter.import_from_string(import_str)
    else:
        app = AppImporter.auto_discover()
        if not app:
            raise ImportError(
                "Cannot auto-discover Jettask app. "
                "Please specify app location (e.g., 'module:app') "
                "or set JETTASK_APP environment variable."
            )
        return app


"""
TaskRun 模型

对应 task_runs 表，用于存储任务执行记录
"""
from sqlalchemy import Column, String, Integer, Text, TIMESTAMP, Float, Index, ForeignKey
from sqlalchemy.dialects.postgresql import JSONB
from datetime import datetime, timezone
from typing import Optional, Dict, Any

from ..base import Base


class TaskRun(Base):
    """
    任务执行记录表

    存储每次任务执行的状态、结果和执行时间等信息
    """
    __tablename__ = 'task_runs'

    stream_id = Column(Text, primary_key=True, comment='Redis Stream 事件ID，唯一标识一个任务')
    started_at = Column(TIMESTAMP(timezone=True), primary_key=True, nullable=False, comment='开始执行时间（TIMESTAMP 类型），分区键')

    task_name = Column(Text, nullable=True, comment='任务名称')

    status = Column(String(50), nullable=True, comment='任务状态 (pending/running/success/failed/retrying)')

    result = Column(JSONB, nullable=True, comment='任务执行结果')
    error = Column(Text, nullable=True, comment='错误信息（如果失败）')

    completed_at = Column(TIMESTAMP(timezone=True), nullable=True, comment='完成时间（TIMESTAMP 类型）')

    retries = Column(Integer, nullable=True, default=0, comment='重试次数')

    duration = Column(Float, nullable=True, comment='执行时长（秒）')

    consumer = Column(Text, nullable=True, comment='执行该任务的消费者ID')

    created_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        default=lambda: datetime.now(timezone.utc),
        comment='记录创建时间'
    )
    updated_at = Column(
        TIMESTAMP(timezone=True),
        nullable=False,
        default=lambda: datetime.now(timezone.utc),
        onupdate=lambda: datetime.now(timezone.utc),
        comment='记录更新时间'
    )

    __table_args__ = (
        Index('idx_task_runs_status', 'status'),
        Index('idx_task_runs_started_at', 'started_at'),
        Index('idx_task_runs_completed_at', 'completed_at'),
        Index('idx_task_runs_created_at', 'created_at'),
    )

    def to_dict(self) -> Dict[str, Any]:
        return {
            'stream_id': self.stream_id,
            'task_name': self.task_name,
            'status': self.status,
            'result': self.result,
            'error': self.error,
            'started_at': self.started_at.isoformat() if self.started_at else None,
            'completed_at': self.completed_at.isoformat() if self.completed_at else None,
            'retries': self.retries,
            'duration': self.duration,
            'consumer': self.consumer,
            'created_at': self.created_at.isoformat() if self.created_at else None,
            'updated_at': self.updated_at.isoformat() if self.updated_at else None,
        }

    def __repr__(self) -> str:
        return f"<TaskRun(stream_id='{self.stream_id}', status='{self.status}')>"



#!/usr/bin/env python
"""时间同步模块

提供与 Redis 服务器时间同步的功能，避免多个 worker 因系统时间不一致导致的限流问题。
在启动时一次性校准与 Redis 服务器的时间差，后续使用本地时间加上时间差来获得标准时间。
"""

import time
import logging
from typing import Optional
import redis.asyncio as aioredis

logger = logging.getLogger(__name__)


class TimeSync:
    """时间同步类

    在初始化时与 Redis 服务器时间进行一次校准，计算时间差。
    后续所有时间获取都使用 本地时间 + 时间差 的方式，避免频繁访问 Redis。

    使用方式:
        # 初始化（异步）
        time_sync = TimeSync()
        await time_sync.sync(redis_client)

        # 获取标准时间
        now = time_sync.time()
    """

    def __init__(self):
        self._offset = 0.0  
        self._synced = False  
        self._sync_timestamp = 0.0  

    async def sync(self, redis_client: aioredis.Redis, retry: int = 3) -> bool:
        for attempt in range(retry):
            try:
                local_before = time.time()

                redis_time = await redis_client.time()

                local_after = time.time()

                network_delay = (local_after - local_before) / 2.0

                redis_timestamp = float(redis_time[0]) + float(redis_time[1]) / 1_000_000.0

                estimated_local_time = local_before + network_delay

                self._offset = redis_timestamp - estimated_local_time
                self._sync_timestamp = time.time()
                self._synced = True

                logger.debug(
                    f"时间同步成功: offset={self._offset:.6f}s, "
                    f"network_delay={network_delay*1000:.2f}ms, "
                    f"redis_time={redis_timestamp:.6f}, "
                    f"local_time={estimated_local_time:.6f}"
                )

                if abs(self._offset) > 1.0:
                    logger.warning(
                        f"本地时间与 Redis 服务器时间差异较大: {self._offset:.2f}s，"
                        f"建议检查系统时间配置"
                    )

                return True

            except Exception as e:
                logger.error(f"时间同步失败 (attempt {attempt + 1}/{retry}): {e}")
                if attempt < retry - 1:
                    await asyncio.sleep(0.1)

        logger.error("时间同步失败，使用本地时间")
        self._synced = False
        return False

    def time(self) -> float:
        if not self._synced:
            return time.time()

        return time.time() + self._offset

    def is_synced(self) -> bool:
        return self._synced

    def get_offset(self) -> float:
        return self._offset

    def get_sync_info(self) -> dict:
        return {
            'synced': self._synced,
            'offset': self._offset,
            'sync_timestamp': self._sync_timestamp,
            'time_since_sync': time.time() - self._sync_timestamp if self._synced else None,
        }


_global_time_sync: Optional[TimeSync] = None


def get_time_sync() -> TimeSync:
    global _global_time_sync
    if _global_time_sync is None:
        _global_time_sync = TimeSync()
    return _global_time_sync


async def init_time_sync(redis_client: aioredis.Redis) -> TimeSync:
    time_sync = get_time_sync()
    await time_sync.sync(redis_client)
    return time_sync


import asyncio



"""
任务级限流器

组合QPS和并发限流，提供统一的任务级限流接口
"""

import asyncio
import logging
import time
from redis.asyncio import Redis
from typing import Dict, Optional, List, Tuple, Any, TYPE_CHECKING

from .qps_limiter import QPSRateLimiter
from .concurrency_limiter import ConcurrencyRateLimiter
from .config import RateLimitConfig, QPSLimit, ConcurrencyLimit, parse_rate_limit_config

logger = logging.getLogger('app')


class TaskRateLimiter:
    """任务级别的限流器

    根据配置类型自动选择合适的限流器实现：
    - QPSLimit: 使用本地滑动窗口 + Redis 协调配额
    - ConcurrencyLimit: 使用 Redis 分布式信号量
    """

    def __init__(
        self,
        redis_client: Redis,
        task_name: str,
        worker_id: str,
        config: RateLimitConfig,
        redis_prefix: str = "jettask",
        sync_interval: float = 5.0,
        worker_state_manager = None
    ):
        self.redis = redis_client
        self.task_name = task_name
        self.worker_id = worker_id
        self.redis_prefix = redis_prefix
        self.global_config = config
        self.sync_interval = sync_interval
        self.worker_state_manager = worker_state_manager

        self.rate_limit_key = f"{redis_prefix}:RATE_LIMIT:CONFIG:{task_name}"

        if isinstance(config, QPSLimit):
            self.limiter = QPSRateLimiter(
                qps=config.qps,
                window_size=config.window_size,
                worker_id=worker_id
            )
            self.mode = 'qps'
            self._sync_task: Optional[asyncio.Task] = None
            self._running = False
        elif isinstance(config, ConcurrencyLimit):
            self.limiter = ConcurrencyRateLimiter(
                redis_client=redis_client,
                task_name=task_name,
                worker_id=worker_id,
                max_concurrency=config.max_concurrency,
                redis_prefix=redis_prefix
            )
            self.mode = 'concurrency'
            self._sync_task = None
            self._running = False
        else:
            raise ValueError(f"Unsupported config type: {type(config)}")

        self.stats = {
            'syncs': 0,
            'last_sync_time': 0,
            'current_worker_count': 0,
        }

    async def start(self):
        if self._running:
            logger.warning(f"TaskRateLimiter for {self.task_name} already running")
            return

        logger.debug(
            f"Starting TaskRateLimiter for task={self.task_name}, "
            f"config={self.global_config}, mode={self.mode}"
        )

        await self._initialize_redis()

        if self.mode == 'qps':
            self._running = True
            self._sync_task = asyncio.create_task(self._sync_loop())

            if self.worker_state_manager:
                self.worker_state_manager.register_callback(self._on_worker_state_change)
                logger.debug(f"Registered worker state change listener for {self.task_name}")

        logger.debug(f"TaskRateLimiter started for {self.task_name}")
    def __delattr__(self, name):
        self.stop()
        
    async def stop(self):
        
        print(f'准备退出限流器 {self.task_name} {self.mode}')
        if not self._running and self.mode == 'qps':
            return

        logger.debug(f"Stopping TaskRateLimiter for {self.task_name}")

        if self.mode == 'qps':
            self._running = False

            if self.worker_state_manager:
                self.worker_state_manager.unregister_callback(self._on_worker_state_change)

            if self._sync_task:
                self._sync_task.cancel()
                try:
                    await self._sync_task
                except asyncio.CancelledError:
                    pass
        elif self.mode == 'concurrency':
            if hasattr(self.limiter, 'stop'):
                try:
                    await self.limiter.stop()
                    logger.debug(f"Concurrency limiter stopped and locks released for {self.task_name}")
                except Exception as e:
                    logger.error(f"Error stopping concurrency limiter for {self.task_name}: {e}")

        logger.debug(f"TaskRateLimiter stopped for {self.task_name}")

    async def _initialize_redis(self):
        try:
            exists = await self.redis.exists(self.rate_limit_key)
            if not exists:
                config_dict = self.global_config.to_dict()
                await self.redis.hset(self.rate_limit_key, mapping=config_dict)
                logger.debug(
                    f"Initialized rate limit config for {self.task_name}: {self.global_config}"
                )

            index_key = f"{self.redis_prefix}:RATE_LIMIT:INDEX"
            await self.redis.sadd(index_key, self.task_name)
            logger.debug(f"Added {self.task_name} to rate limit index")

        except Exception as e:
            logger.error(f"Failed to initialize Redis: {e}")

    async def _on_worker_state_change(self, state_data: dict):
        if self.mode == 'qps':
            logger.debug(f"[{self.task_name}] Worker state changed: {state_data}")
            await self._sync_quota()

    async def _sync_quota(self):
        if self.mode != 'qps':
            return

        try:
            config_dict = await self.redis.hgetall(self.rate_limit_key)
            if not config_dict:
                logger.warning(f"No config found in Redis for {self.task_name}")
                return

            config_dict = {
                k.decode() if isinstance(k, bytes) else k:
                v.decode() if isinstance(v, bytes) else v
                for k, v in config_dict.items()
            }

            global_config = parse_rate_limit_config(config_dict)
            if not isinstance(global_config, QPSLimit):
                logger.error(f"Invalid QPS config for {self.task_name}: {config_dict}")
                return

            worker_ids = [self.worker_id]
       
            workers = await self.worker_state_manager(self.task_name, only_alive=True)
            if workers:
                worker_ids = sorted(list(workers))
             

            worker_count = len(worker_ids)

            await self._allocate_quotas(global_config.qps, worker_ids)

            quota_key = f"quota:{self.worker_id}"
            quota_str = await self.redis.hget(self.rate_limit_key, quota_key)

            if quota_str:
                quota_value = int(quota_str)
            else:
                quota_value = global_config.qps // worker_count if worker_count > 0 else global_config.qps

            await self.limiter.update_limit(quota_value)

            self.stats['syncs'] += 1
            self.stats['last_sync_time'] = time.time()
            self.stats['current_worker_count'] = worker_count

        except Exception as e:
            logger.error(f"Quota sync error: {e}")

    async def _sync_loop(self):
        while self._running:
            try:
                await self._sync_quota()
                await asyncio.sleep(self.sync_interval)
            except asyncio.CancelledError:
                break
            except Exception as e:
                logger.error(f"Sync loop error: {e}")
                await asyncio.sleep(1)

    async def _allocate_quotas(self, total_qps: int, worker_ids: list):
        lock_key = f"{self.rate_limit_key}:lock"

        try:
            async with self.redis.lock(lock_key, timeout=3, blocking_timeout=2):
                worker_count = len(worker_ids)
                base_quota = total_qps // worker_count
                remainder = total_qps % worker_count

                allocations = {}
                for i, worker_id in enumerate(worker_ids):
                    quota = base_quota + (1 if i < remainder else 0)
                    allocations[f"quota:{worker_id}"] = quota

                if allocations:
                    await self.redis.hset(self.rate_limit_key, mapping=allocations)

                await self._cleanup_stale_quotas(worker_ids)

                logger.debug(f"[ALLOCATE] Allocated quotas for {worker_count} workers: {allocations}")

        except asyncio.TimeoutError:
            logger.debug(f"[ALLOCATE] Failed to acquire lock, skipping allocation")
        except Exception as e:
            logger.error(f"[ALLOCATE] Error allocating quotas: {e}")

    async def _cleanup_stale_quotas(self, active_worker_ids: list):
        try:
            all_fields = await self.redis.hkeys(self.rate_limit_key)
            active_quota_keys = {f"quota:{wid}" for wid in active_worker_ids}

            to_delete = []
            for field in all_fields:
                if isinstance(field, bytes):
                    field = field.decode('utf-8')

                if field.startswith("quota:") and field not in active_quota_keys:
                    to_delete.append(field)

            if to_delete:
                await self.redis.hdel(self.rate_limit_key, *to_delete)
                logger.debug(f"[CLEANUP] Removed {len(to_delete)} stale quotas")

        except Exception as e:
            logger.error(f"[CLEANUP] Error cleaning up quotas: {e}")

    async def acquire(self, timeout: float = 10.0) -> Optional[str]:
        result = await self.limiter.acquire(timeout=timeout)
        if self.mode == 'qps':
            return True if result else None
        else:
            return result  

    async def release(self, task_id: Optional[str] = None):
        if self.mode == 'concurrency' and task_id is None:
            raise ValueError("ConcurrencyLimit requires task_id for release")

        if self.mode == 'concurrency':
            await self.limiter.release(task_id)
        else:
            pass

    async def try_acquire(self) -> bool:
        return await self.limiter.try_acquire()

    def get_stats(self) -> dict:
        return {
            **self.stats,
            'task_name': self.task_name,
            'worker_id': self.worker_id,
            'config': str(self.global_config),
        }

    async def __aenter__(self):
        task_id = await self.acquire()
        if task_id is None:
            raise TimeoutError("Failed to acquire rate limit slot")
        self._current_task_id = task_id
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if hasattr(self, '_current_task_id'):
            task_id = self._current_task_id
            delattr(self, '_current_task_id')
            if self.mode == 'concurrency':
                await self.release(task_id)
        return False  





__all__ = ['TaskRateLimiter']



"""
设置模块 - 系统配置
提供轻量级的路由入口，业务逻辑在 SettingsService 中实现
"""
from fastapi import APIRouter, HTTPException
from typing import Dict, Any
import logging
import traceback

from jettask.schemas import SystemSettingsResponse, DatabaseStatusResponse
from jettask.webui.services.settings_service import SettingsService

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/settings", tags=["settings"])



@router.get(
    "/system",
    summary="获取系统配置信息",
    description="获取系统级别的配置信息，包括 API 版本、服务名称、运行环境、数据库状态等",
    response_model=SystemSettingsResponse,
    responses={
        200: {
            "description": "成功返回系统配置信息"
        },
        500: {
            "description": "服务器内部错误",
            "content": {
                "application/json": {
                    "example": {
                        "detail": "获取系统配置失败: Configuration error"
                    }
                }
            }
        }
    }
)
async def get_system_settings() -> Dict[str, Any]:
    try:
        return SettingsService.get_system_settings()
    except Exception as e:
        logger.error(f"获取系统配置失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


@router.get(
    "/database-status",
    summary="检查数据库连接状态",
    description="检查数据库（PostgreSQL/MySQL）的连接状态和性能指标",
    response_model=DatabaseStatusResponse,
    responses={
        200: {
            "description": "成功返回数据库状态"
        },
        500: {
            "description": "服务器内部错误或数据库连接失败",
            "content": {
                "application/json": {
                    "example": {
                        "detail": "数据库状态检查失败: Connection refused"
                    }
                }
            }
        }
    }
)
async def check_database_status() -> Dict[str, Any]:
    try:
        return await SettingsService.check_database_status()
    except Exception as e:
        logger.error(f"数据库状态检查失败: {e}")
        raise HTTPException(status_code=500, detail=str(e))


__all__ = ['router']